#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
The registration application provides functionality to differenciate
between anonymous, regular and staff users.

Anonymous users can access only minimal functionality:

- perform search sessions

Regular users can access additional functionality:

- see the training examples
- perform evaluation tests

Staff users can access additional functionality:

- manage (create/delete/modify) the User, Task, Config models
- view the statistics of the system performance
- watch the test search sessions saved in the database

There is some functionality for managing user groups, e.g. expert users,
but there is not much use of it for now.
"""

###########################################################################
# IMPORTS

from django.conf import settings
from django.db import models
from django.contrib.auth.models import Group as DjangoGroup
from django.contrib.auth.models import User as DjangoUser
#from django.contrib.auth.models import Permission
#from django.contrib.contenttypes.models import ContentType

###########################################################################
# GLOBALS

GENDER_CHOICES = (
    ('M', 'Man'),
    ('F', 'Woman'),
)

###########################################################################
# CLASS

class Group(DjangoGroup):
    """
    Group model is a sub-class of the DjangoGroup model.
    """

    class Meta:
        permissions = (
            ('is_expert', 'Expert Users'),
            #('is_premium', 'Premium Users'),
            #('is_special', 'Special Users'),
        )

###########################################################################
# CLASS

class User(DjangoUser):
    """
    User model is a sub-class of the DjangoUser model.
    """

    age = models.PositiveSmallIntegerField('age',
                                           db_column='age')

    gender = models.CharField('gender',
                              max_length=1,
                              choices=GENDER_CHOICES,
                              db_column='gender')

    class Meta:
        db_table = settings.TABLE_USER
        managed = False

    #######################################################################

    def add_to_group_expert(self):

        # get the group
        group = Group.objects.get(name='Expert Users')
        #group = Group.objects.filter(name='Expert Users')
        #if group:
        #    group = group[0]
        #else:
        #    # example:  create a new group
        #    group = Group()
        #    group.name = 'Expert Users'
        #    group.save()

        # get the permission
        #permission = Permission.objects.get(codename='is_expert')
        #permission = Permission.objects.filter(codename='is_expert')
        #if permission:
        #    permission = permission[0]
        #else:
        #    # example: create a new permission
        #    somemodel_ct = ContentType.objects.get(app_label='registration', model='Group')
        #    permission = Permission (name='Expert Users',
        #                             codename='is_expert',
        #                             content_type=somemodel_ct)
        #    permission.save()

        # add permissions to the group
        #group.permissions.add(permission)
        #group.permissions = [permission]

        self.groups.add(group)
        #print self.id, self.has_perm('registration.is_expert')

    def remove_from_group_expert(self):

        # get the group
        group = Group.objects.get(name='Expert Users')
        self.groups.remove(group)
