#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
"""

###########################################################################
# IMPORTS

import logging
import sys

from django.conf import settings
from django.core.urlresolvers import reverse

from retrieval_libs.dblayer.DbAccess import DbAccess

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

# implemented as a singleton
dbaccess = DbAccess()

###########################################################################
# FUNCTION

def current_page(request):
    """
    Provides the url of the current page.

    HINT: When switching to another database, it should stay on the same page,
          but it cannot just re-apply the specific requests that contain some
          incompatible id/data/status.
    """

    current_url = request.get_full_path()

    if current_url:

        if current_url == '/':
            # default page
            current_page = settings.LOGIN_REDIRECT_URL

        elif 'registration' in current_url:
            # unchanged
            current_page = current_url

        elif 'search' in current_url:
            # retrieve the active_mode
            active_mode = request.session.get('active_mode', None)

            if active_mode == 'search':
                current_page = reverse('retrieval:search:index')
            elif active_mode == 'test':
                current_page = reverse('retrieval:main:test')
            elif active_mode == 'train':
                current_page = reverse('retrieval:search:train')
            else:
                # unchanged
                current_page = current_url
                logger.warning('    UNKNOWN URL %s' % current_url)

        elif 'watch' in current_url:
            # retrieve the active_mode
            active_mode = request.session.get('active_mode', None)

            #if active_mode == 'watch':
            if 'example' in current_url:
                current_page = reverse('retrieval:watch:example')
            else:
                current_page = reverse('retrieval:watch:index')

        elif 'statistics' in current_url:
            current_page = reverse('retrieval:statistics:index')

        elif 'help' in current_url:
            current_page = reverse('retrieval:help:index')

        elif 'manage' in current_url:
            if 'users' in current_url:
                current_page = reverse('retrieval:manage:users')
            elif 'tasks' in current_url:
                current_page = reverse('retrieval:manage:tasks')
            elif 'configs' in current_url:
                current_page = reverse('retrieval:manage:configs')
            elif 'sessions' in current_url:
                current_page = reverse('retrieval:manage:sessions')
            elif current_url == reverse('retrieval:manage:index'):
                current_page = current_url
            else:
                # unchanged
                current_page = current_url
                logger.warning('    UNKNOWN URL %s' % current_url)

        else:
            # unchanged
            current_page = current_url
            #sys.stderr.write('\nUNKNOWN URL %s\n' % current_url)
            #sys.exit(1)

        to_return = {'current_page': current_page}

    else:
        to_return = {}

    return to_return

###########################################################################

def active_db(request):
    """
    Provides the active database.
    """

    active_db = request.session.get('active_db', None)

    to_return = {}
    if active_db is not None:
        to_return['active_db'] = active_db

    return to_return

###########################################################################

def img_url(request):
    """
    Provides media-related context variables.
    """

    ALL_DB = settings.ALL_DB

    active_db = request.session.get('active_db', None)

    to_return = {}
    if active_db is not None:
        data = ALL_DB[active_db]
        IMG_URL = data.get('IMG_URL', None)
        if IMG_URL is not None:
            to_return['IMG_URL'] = IMG_URL

    return to_return

###########################################################################

def dbinfo(request):
    """
    Provides display info about the database.
    """

    u = request.user

    active_mode = request.session.get('active_mode', None)

    if active_mode == 'test' and not u.is_staff:
        to_return = {}
    else:
        to_return = dbaccess.displayinfo()

    return to_return
