#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
"""

###########################################################################
# IMPORTS

from django import http
from django.conf import settings
from django.shortcuts import render_to_response
from django.template import Context, RequestContext, loader
from django.views.decorators.cache import never_cache

from retrieval.helpers.UserTestController import nb_sessions_done

###########################################################################
# GLOBALS

ITER_MAX = settings.ITER_MAX

###########################################################################

@never_cache
def index(request):
    """
    """

    user = request.user

    context_data = {}
    context_data['iter_max'] = ITER_MAX
    context_data['nb_sessions'] = nb_sessions_done(user)

    return render_to_response(
        'retrieval/help/index.html',
        context_data,
        context_instance=RequestContext(request))

###########################################################################

def handler404(request, template_name='404.html'):
    """
    Handler for the page not found error 404.
    """

    t = loader.get_template(template_name)
    requestcontext = RequestContext(request)
    return http.HttpResponseNotFound(t.render(requestcontext))

def handler500(request, template_name='500.html'):
    """
    Handler for the server error 500.
    """

    t = loader.get_template(template_name)
    requestcontext = RequestContext(request)
    return http.HttpResponseServerError(t.render(requestcontext))
