#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Provides the means to go through the iterations of the stored sessions
"""

###########################################################################
# IMPORTS

import logging
import sys
from time import time as now

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

###########################################################################
# CLASS

class Watch(object):
    """
    Provides the means to go through the iterations of the stored sessions.
    """

    #######################################################################
    # main functionality

    def init(self, session, data):
        """
        Initialize the object.
        """

        self.session = session

        self.iteration = data.get('iteration', 1)

        self.rankdata = None
        self.rankdata_0 = None
        self.dispdata = None
        self.tracedata = None

    def update(self):
        """
        Perform the watch in the database, and update the CLASS data.
        """

        dispfdbkH = self.session.dispfdbkH
        #traceH = self.session.traceH

        self.iteration = self.iteration % (len(dispfdbkH))

        self.rankdata = None
        self.rankdata_0 = None

        dispdata = dispfdbkH[self.iteration][0]
        if dispdata is not None:
            self.dispdata = dispdata
        else:
            self.dispdata = None

        #if traceH:
        #    self.tracedata = traceH[self.iteration]
        #else:
        self.tracedata = None

    def displayinfo(self):
        """
        Prepare the display information.
        """

        info = self.session.displayinfo(self.iteration, watching=True)

        return info
