#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
"""

###########################################################################
# IMPORTS

import sys

from django.conf import settings
from django.db import models

from retrieval_libs.dblayer.DbAccess import DbAccess

###########################################################################
# GLOBALS

# implemented as a singleton
dbaccess = DbAccess()

# the list of databases
ALL_DB = settings.ALL_DB

# the default database
DEFAULT_DB = settings.DEFAULT_DB

FIGURES = settings.FIGURES

###########################################################################
# FUNCTIONS

def setup_db(request, new_db=None, init_db=True):
    """
    Set up the given database.

    If new_db = None, preserve the existing cache. Otherwise, refresh it.
    If init_db = True, initialize the DbAccess singleton.
    """

    # retrieve the active_db from the cache
    active_db = request.session.get('active_db', None)
    if new_db is None:
        new_db = active_db

    # determine the new active_db
    if active_db is None:
        active_db = DEFAULT_DB
        buffer_is_valid = False
    elif new_db != active_db:
        active_db = new_db
        buffer_is_valid = False
    else: # new_db == active_db
        buffer_is_valid = True

    if active_db not in ALL_DB:
        sys.stderr.write('\nUNKNOWN DB %s\n' % active_db)
        sys.exit(1)

    # update the session buffers
    if not buffer_is_valid:

        request.session['active_db'] = active_db

        if 'session' in request.session:
            del request.session['session']

        if 'dispdata' in request.session:
            del request.session['dispdata']

        if 'config' in request.session:
            del request.session['config']

        if 'task' in request.session:
            del request.session['task']

        if 'statistics' in request.session:
            #del request.session['statistics']
            buffer_statistics = request.session['statistics']
            if 'tasks' in buffer_statistics:
                del buffer_statistics['tasks']
            if 'configs' in buffer_statistics:
                del buffer_statistics['configs']

            # initialize the statistics configuration from the settings
            # HINT: If settings are empty, all existing TASKS and CONFIGS
            #       in the database will be considered.
            #buffer_statistics['tasks'] = ', '.join(map(str, ALL_DB[active_db]['TASKS']))
            buffer_statistics['configs'] = ', '.join(map(str, ALL_DB[active_db]['CONFIGS']))

    if init_db:

        user = request.user
        username = user.username
        if username is not None:
            FIGURES['username'] = username

        # initialize the DbAccess singleton
        dbaccess.init(ALL_DB, FIGURES, active_db)

    return request

###########################################################################

def create_model(name, super_, module, app_label=None, fields=None, options=None):
    """
    Create the specific model classtype on-the-fly.
    """

    class Meta:
        pass

    # update Meta with the app_label
    if app_label is not None:
        setattr(Meta, 'app_label', app_label)

    # update Meta with the options
    if options is not None:
        for key, value in options.iteritems():
            setattr(Meta, key, value)

    # set up a dictionary to simulate the class declarations
    attrs = {'__module__': module, 'Meta': Meta}

    # add the fields
    if fields is not None:
        attrs.update(fields)

    # create the class, which automatically triggers ModelBase processing
    model = type(name, super_, attrs)

    return model
