#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
"""

###########################################################################
# IMPORTS

from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.contrib.auth.decorators import login_required
from django.views.decorators.cache import never_cache

from retrieval.helpers.shortcuts import setup_db
from retrieval.helpers.UserTestController import assign_db

###########################################################################

@never_cache
def index(request, new_db):
    """
    Switch to the given database and redirect to the same page.
    """

    request = setup_db(request, new_db=new_db, init_db=False)

    redirect_to = request.REQUEST.get('page', None)
    if redirect_to is None:
        # default redirect
        redirect_to = reverse('retrieval:search:index')

    return HttpResponseRedirect(redirect_to)

###########################################################################

@login_required
@never_cache
def test(request):
    """
    Assign the database and redirect to the corresponding testing.
    """

    # assign a database to the current user
    new_db = assign_db(request.user.id)

    setup_db(request, new_db=new_db, init_db=False)

    redirect_to = reverse('retrieval:search:test')

    return HttpResponseRedirect(redirect_to)
