#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Provides the forms corresponding to the models.
"""

###########################################################################
# IMPORTS

from django.forms import ModelForm

from retrieval.helpers.shortcuts import create_model
from retrieval.models import get_task_model

###########################################################################
# FUNCTION

def get_task_form(active_db):
    """
    Creates the corresponding form.
    """

    Task = get_task_model(active_db)

    options = {'model': Task,}

    form = create_model('TaskForm',
                        super_=(_TaskForm,),
                        module=_TaskForm.__module__,
                        options=options)

    return form

###########################################################################
# CLASS

class _TaskForm(ModelForm):
    """
    Super class with customized field display settings.
    """

    def __init__(self, *args, **kwargs):

        # Call to ModelForm constructor
        super(_TaskForm, self).__init__(*args, **kwargs)

        self.fields['target'].widget.attrs['cols'] = 20
        self.fields['target'].widget.attrs['rows'] = 5
