#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
"""

###########################################################################
# IMPORTS

#import logging

from django.http import HttpResponseForbidden

from retrieval.helpers.shortcuts import setup_db

###########################################################################
# GLOBALS

#logger = logging.getLogger('retrieval')

BOT_NAMES = ['Googlebot', 'Slurp', 'Twiceler', 'KaloogaBot', 'YodaoBot',
             'Baiduspider', 'Speedy Spider', 'DotBot',
             'googlebot', 'msnbot', 'nagios']

# HINT: add param to url(r'..', include('..urls'), {'deny_crawlers': True})
#PARAM_NAME = 'deny_crawlers'

###########################################################################
# CLASS

class DbSetup(object):
    """
    Set up the system for the current database.
    """

    def process_request(self, request):
        """
        """

        request = setup_db(request)

###########################################################################
# CLASS

class CrawlerBlocker(object):
    """
    Block the requests coming from crawling robots.
    """

    def process_request(self, request):

        user_agent = request.META.get('HTTP_USER_AGENT', None)
        #logger.info('HTTP_USER_AGENT: %s' % user_agent)

        if not user_agent:
            # check if it is a request from the django development server
            # or a dummy request of the django test environment
            remote_addr = request.META.get('REMOTE_ADDR', None)
            #logger.debug('REMOTE_ADDR: %s' % remote_addr)
            if remote_addr == '127.0.0.1':
                request.is_crawler = False
                return

            return HttpResponseForbidden(
                    'Request has no HTTP_USER_AGENT field.')

        request.is_crawler = False
        for botname in BOT_NAMES:
            if botname in user_agent:
                request.is_crawler = True

    def process_view(self, request, view_func, view_args, view_kwargs):

        #if PARAM_NAME in view_kwargs:
        #    if view_kwargs[PARAM_NAME]:
        #        del view_kwargs[PARAM_NAME]

        if request.is_crawler:
            # clean the session cache
            # HINT: the session cache of the request becomes orphan memory,
            # so it should be minimized.
            for key in request.session.keys():
                del request.session[key]

            return HttpResponseForbidden(
                    'Request is denied for crawling robots.'
                    'Check your HTTP_USER_AGENT field.')
