#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
"""

###########################################################################
# IMPORTS

import logging
from time import time as now

from django.conf import settings
from django.contrib.auth.decorators import login_required, permission_required
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.decorators.cache import never_cache

from retrieval.helpers.UserTestController import nb_sessions_done, \
                                                 nb_sessions_per_test
from retrieval.models import get_statistics_model
from retrieval.statistics.forms import get_statistics_form

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

#active_db
#buffer_statistics

###########################################################################

# get the data stored in the session instance
def _get_state(request):

    global active_db
    global buffer_statistics

    active_db = request.session.get('active_db', None)

    if request.POST.get('update_data', None):
        buffer_statistics = request.session.get('statistics', {})
    elif request.POST.get('update_imgs', None):
        buffer_statistics = request.session.get('statistics', {})
    else:
        #buffer_statistics = {}
        buffer_statistics = request.session.get('statistics', {})

# store data in the session instance
def _set_state(request):

    if buffer_statistics:
        request.session['statistics'] = buffer_statistics
    else:
        if 'statistics' in request.session:
            del request.session['statistics']

###########################################################################

def _statistics(request):

    global buffer_statistics

    start_time = now()

    _get_state(request)
    user = request.user
    username = user.username

    Statistics = get_statistics_model(active_db)
    statistics = Statistics()

    statistics.init()
    statistics.set_data(buffer_statistics)

    form = None

    StatisticsForm = get_statistics_form(active_db)

    if request.POST.get('update_data', None): # If the form has been submitted...
        if statistics is None:
            form = StatisticsForm(request.POST) # Bound the form to the POST data
        else:
            form = StatisticsForm(request.POST, instance=statistics) # Bound the form to the POST data

        if form.is_valid(): # If all validation rules pass...
            statistics = form.save(commit=False)
            statistics.set_data()
            form = None
        else:
            message = 'Correct the form fields'

    elif request.POST.get('update_imgs', None): # If the form has been submitted...
        input_figids = []
        for k in request.POST.keys():
            if k[0:3] == 'im_' and request.POST[k] == 'on':
                input_figids.append(int(k[3:]))

        input_figids = sorted(input_figids)
        statistics.figids = input_figids

    if form is None:
        if statistics is None:
            form = StatisticsForm() # An unbound form
            message = ''#Selected data'
        else:
            form = StatisticsForm(instance=statistics) # Bound the form to the TASK data
            message = ''#Modify the selected data'

    imurls, imstat = statistics.update_images()
    buffer_statistics = statistics.get_data()
    _set_state(request)

    stop_time = now()
    logger.info('    DONE in %f seconds' % (stop_time - start_time))

    return form, message, imurls, imstat

###########################################################################

def _statistics_user(u):

    username = u.username
    u_dbid = u.id

    Statistics = get_statistics_model(active_db)
    statistics = Statistics()

    statistics.init()
    statistics.set_data()

    imurls = statistics.update_image_user(u_dbid, username)

    return imurls

###########################################################################

@login_required
#@permission_required('registration.is_expert')
@never_cache
def index(request):

    _get_state(request)

    user = request.user

    context_data = {}
    context_data['nb_sessions'] = nb_sessions_done(user)

    if user.has_perm('registration.is_expert'):
        statistics_form, statistics_message, imurls, imstat = _statistics(request)
        context_data['statistics_form'] = statistics_form
        context_data['statistics_message'] = statistics_message
        context_data['imurls'] = imurls
        context_data['imstat'] = imstat

        return render_to_response(
            'retrieval/statistics/index.html',
            context_data,
            context_instance=RequestContext(request))

    else:
        context_data['nb_sessions_per_test'] = nb_sessions_per_test()
        context_data['imurls'] = _statistics_user(user)

        return render_to_response(
            'retrieval/statistics/user_statistics.html',
            context_data,
            context_instance=RequestContext(request))
