#! /usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Computes statistics of the image similarity distances.
"""

###########################################################################
# IMPORTS

import logging

from django.core.management import setup_environ
import heat.settings.maintenance as settings
setup_environ(settings)

from retrieval_libs.dblayer.DbAccess import DbAccess
from retrieval_libs.dbutils.dbimg_statistics import DbImg_Statistics

from retrieval.models import get_config_model

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

# implemented as a singleton
dbaccess = DbAccess()

ALL_DB = settings.ALL_DB
FIGURES = settings.FIGURES

# HINT: keep it commented out, to avoid any execution by mistake
#ACTIVE_DB = 'ImageNet_60K'

#ACTIVE_CONFIG_ID = ALL_DB[ACTIVE_DB]['DEFAULT_CONFIG']
ACTIVE_CONFIG_ID = 1

PERCENTAGE = 10.0

###########################################################################
# main functionality

def compute_daverage(active_db, config_id):
    """
    Compute statistics of the image similarity distances.
    """

    # retrieve the config
    Config = get_config_model(active_db)
    c = Config.objects.get(pk=config_id)
    c.init()
    thargs = c.SYS_THARGS[0]
    weights = c.SYS_WEIGHTS[0]

    dbimg_stats = DbImg_Statistics()
    dbimg_stats.init(dbaccess, thargs, weights)

    daverage = dbimg_stats.daverage(0)

    return daverage

def compute_statistics(active_db, config_id, percentage):
    """
    Compute statistics of the image similarity distances.
    """

    # retrieve the config
    Config = get_config_model(active_db)
    c = Config.objects.get(pk=config_id)
    c.init()
    thargs = c.SYS_THARGS[0]
    weights = c.SYS_WEIGHTS[0]

    dbimg_stats = DbImg_Statistics()
    dbimg_stats.init(dbaccess, thargs, weights)

    #dbimg_stats.histogram(0)
    dbimg_stats.threshold(0, percentage=percentage)

###########################################################################
# the main if we test/execute this module

if __name__ == '__main__':

    dbaccess.init(ALL_DB, FIGURES, ACTIVE_DB)

    compute_statistics(ACTIVE_DB, ACTIVE_CONFIG_ID, PERCENTAGE)
