#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2010                   - IDIAP Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Updates the urls validity flags.
"""

###########################################################################

import logging
import os
from numpy import int32, array
import string
import sys
import urllib2

from django.core.management import setup_environ
import heat.settings.maintenance as settings
setup_environ(settings)

from retrieval_libs.dblayer.DbAccess import DbAccess

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

# implemented as a singleton
dbaccess = DbAccess()

ALL_DB = settings.ALL_DB
FIGURES = settings.FIGURES

# HINT: keep it commented out, to avoid any execution by mistake
#ACTIVE_DB = 'ImageNet_1M'

BLOCKLN = 60000
SUBBLOCKLN = 1000

###########################################################################

def _i_block(num, BLOCKLN):
    """
    Get the (indexes of the) block for processing.
    """

    if os.getenv('SGE_TASK_ID') is None:
        # data is processed in one block
        i_from = 1
        i_to = num + 1
    else:
        # data is processed in blocks, one block per job
        nb_blocks = num / BLOCKLN + 1

        job_id = int(os.getenv('SGE_TASK_ID'))
        job_index = job_id - 1

        i_from = min(job_index * BLOCKLN + 1, num + 1)
        i_to = min(job_index * BLOCKLN + BLOCKLN, num + 1)

        stepsize = int(os.getenv('SGE_TASK_STEPSIZE'))
        if stepsize != 1:
            sys.stderr.write('\nSGE_TASK_STEPSIZE %s != 1\n' % stepsize)
            sys.exit(1)

    return i_from, i_to

def _check_url(imurl):
    """
    Check if the url is valid.
    """

    valid = True
    imname_request = None
    imname_response = None
    try:
        request = urllib2.Request(imurl)
        request.get_method = lambda : 'HEAD'

        response = urllib2.urlopen(request, data=None, timeout=4)
        answer = response.geturl()

        imname_request = imurl.split('/')[-1]
        imname_response = answer.split('/')[-1]
        if imname_request != imname_response:
            valid = False
    except:
        valid = False

    if not valid:
        logger.info('VALID %s IMNAME %s != %s' %
                     (valid, imname_request, imname_response))

    return valid

def update_valid_url():
    """
    Update the flags for valid urls.
    """

    dbimg = dbaccess.dbimg()
    num = dbimg.size()

    i_from, i_to = _i_block(num, BLOCKLN)
    logger.info('    PROGRESS INFO %s - %s' % (i_from, i_to))

    # process the data in SUBBLOCKS
    nb_subblocks = (i_to - i_from) / SUBBLOCKLN + 1
    for ii in xrange(0, nb_subblocks, 1):
        ii_from = i_from + ii * SUBBLOCKLN
        ii_to = min(ii_from + SUBBLOCKLN, i_to)

        if ii_from >= ii_to:
            continue

        logger.info('    PROGRESS INFO %s - %s' % (ii_from, ii_to))

        # get the subblock for processing
        dbid = range(ii_from, ii_to, 1)

        imurls = dbimg.im_path2(dbid)
        for imdbid, imvalid, imurl in imurls:

            if imvalid == 0:
                continue

            valid = _check_url(imurl)
            if not valid:
                dbimg.update_valid_url(imdbid, 0)

            logger.debug('DBID %s VALID %s URL %s' % (imdbid, valid, imurl))

def update_valid_url_0():
    """
    Update the flags for valid urls.
    """

    dbimg = dbaccess.dbimg()
    num = dbimg.size()

    i_from, i_to = _i_block(num, BLOCKLN)
    logger.info('    PROGRESS INFO %s - %s' % (i_from, i_to))

    # process the data in SUBBLOCKS
    nb_subblocks = (i_to - i_from) / SUBBLOCKLN + 1
    for ii in xrange(0, nb_subblocks, 1):
        ii_from = i_from + ii * SUBBLOCKLN
        ii_to = min(ii_from + SUBBLOCKLN, i_to)

        if ii_from >= ii_to:
            continue

        logger.info('    PROGRESS INFO %s - %s' % (ii_from, ii_to))

        # get the subblock for processing
        dbid = range(ii_from, ii_to, 1)
        dbid = array(dbid, dtype=int32, order='c')

        featdata = dbimg.featdata(dbid, 0, False)

        for i, idbid in enumerate(dbid):

            ifeat = featdata[0][i,:]
            #print 'ifeat', len(ifeat), ifeat.shape

            # check histogram
            ifeat_sum = ifeat.sum()
            valid = True
            if ifeat_sum < 50:
                valid = False
                dbimg.update_valid_url(idbid, 0)

            logger.debug('DBID %s VALID %s SUM %s' % (idbid, valid, ifeat_sum))

###########################################################################
# the main if we test/execute this module

if __name__ == '__main__':

    dbaccess.init(ALL_DB, FIGURES, ACTIVE_DB)

    #update_valid_url_0()
    update_valid_url()
