#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Provides methods to translate data from run-time to database format
and viceversa.
"""

###########################################################################
# IMPORTS

from numpy import int32, float32, ndarray, array, fromstring
import sys

from retrieval_libs.common.thdist import get_thcoefs

###########################################################################
# FUNCTIONS

def _arrays2str(data):
    """
    """

    stritems = []

    for item in data:
        if item is None:
            stritem = 'None'
        elif isinstance(item, ndarray):
            substritems = []
            for t in xrange(0, item.shape[1], 1):
                substritem = ' '.join(map(str, item[:,t]))
                substritems.append(substritem)
            stritem = ', '.join(substritems)
        elif isinstance(item, (str, unicode)):
            stritem = item
        else:
            print 'item', item
            sys.stderr.write('\nUNKNOWN TYPEx %s\n' % type(item))
            sys.exit(1)

        stritems.append(stritem)

    strdata = '; '.join(stritems)

    return strdata

###########################################################################

def weights2str(data):
    """
    """

    if isinstance(data, ndarray):
        data = [data,]

    strdata = _arrays2str(data)

    return strdata

def str2weights(strdata):
    """
    """

    if strdata == '':
        data = None
    else:
        data = []
        stritems = strdata.split('; ')
        for i, stritem in enumerate(stritems):
            if stritem == 'None':
                item = None
            elif ',' in stritem:
                subitems = []
                strsubitems = stritem.split(', ')
                strsubitemsln = len(strsubitems)
                stritem = ' '.join(strsubitems)
                item = fromstring(stritem, dtype=float32, sep=' ')
                itemln = len(item)
                item.shape = (itemln/strsubitemsln, strsubitemsln)
            else:
                item = stritem

            data.append(item)

    return data

###########################################################################

def consistency2str(data):
    """
    """

    stritems = []

    for i, item in enumerate(data):
        stritem = str(item)

        stritems.append(stritem)

    strdata = '; '.join(stritems)

    return strdata

def str2consistency(strdata):
    """
    """

    if strdata == '':
        data = None
    else:
        data = []
        stritems = strdata.split('; ')
        for i, stritem in enumerate(stritems):
            if '.' in stritem:
                item = float32(stritem)
            else:
                item = stritem

            data.append(item)

    return data

###########################################################################

def thargs2str(data):
    """
    """

    filtered = []
    if isinstance(data[0], ndarray):
        filtered.append(data[0])

    else:
        for item in data:
            if isinstance(item, tuple):
                filtered.append(item[0])
            else:
                filtered.append(item)

    strdata = _arrays2str(filtered)

    return strdata

def str2thargs(strdata):
    """
    """

    if strdata == '':
        data = None
    else:
        data = []
        stritems = strdata.split('; ')
        for stritem in stritems:
            if stritem == 'None':
                item = None
            elif ',' in stritem:
                info0 = stritem.split(', ')
                for i in xrange(0, len(info0), 1):
                    info0[i] = info0[i].split(' ')
                info0 = zip(info0[0], info0[1])
                item = array(info0, dtype=float32, order='c')
                item = (item, get_thcoefs(item))
            else:
                item = stritem

            data.append(item)

    return data

###########################################################################

def dispfdbk2str(data):
    """
    """

    stritems = []
    for item in data:
        strsubitems = []
        for subitem in item:
            if subitem is None:
                strsubitem = 'None'
            elif isinstance(subitem, dict):
                # keep only 'dbid' data
                strsubitem = ' '.join(map(str, subitem['imdbid']))
            else:
                strsubitem = str(subitem)

            strsubitems.append(strsubitem)

        stritem = ', '.join(strsubitems)
        stritems.append(stritem)

    data = '; '.join(stritems)

    return data

def str2dispfdbk(strdata):
    """
    """

    #self.dispfdbkH = eval(self.dispfdbkH)
    if strdata == '':
        data = None
    else:
        items = []
        stritems = strdata.split('; ')
        for stritem in stritems:
            subitems = []
            strsubitems = stritem.split(', ')
            for strsubitem in strsubitems:
                if ' ' in strsubitem:
                    # create dict with 'dbid' data
                    subitem = fromstring(strsubitem, dtype=int32, sep=' ')
                    subitem = {'imdbid': subitem}
                elif strsubitem == 'None':
                    subitem = None
                else:
                    subitem = int32(strsubitem)

                subitems.append(subitem)

            items.append(tuple(subitems))

        data = tuple(items)

    return data

###########################################################################

def target2str(data):
    """
    """

    if data is None:
        strdata = ''
    else:
        strdata = ' '.join(map(str, data))

    return strdata

def str2target(strdata):
    """
    """

    if strdata == '':
        data = None
    else:
        try:
            data = fromstring(strdata, dtype=int32, sep=' ')
        except:
            sys.stderr.write('\nPARSING ERROR "%s"\n' % strdata)
            sys.exit(1)

    return data
