#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Provides the buffer data. Implemented as a very simple singleton.
"""

import logging
import sys

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

###########################################################################
# CLASS

class DataBuffer(object):
    """
    Provides the buffer data. Implemented as a very simple singleton.
    """

    #######################################################################

    __single = None # the one, true Singleton

    def __new__(classtype, *args, **kwargs):
        # Check to see if a __single exists already for this class
        # Compare class types instead of just looking for None so
        # that subclasses will create their own __single objects
        if classtype != type(classtype.__single):
          classtype.__single = object.__new__(classtype, *args, **kwargs)

        return classtype.__single

    #######################################################################

    def __init__(self):
        """
        Initialize the object.
        """

        self.data = None

    #######################################################################

    def set_data(self, data):

        if not isinstance(data, dict):
            sys.stderr.write('\nUNKNOWN TYPE %s\n' % type(data))
            sys.exit(1)

        if self.data is not None:
            sys.stderr.write('\nDATA BUFFER OVERWRITTEN\n')
            sys.exit(1)

        self.data = data

    def get_data(self):

        if self.data is None:
            sys.stderr.write('\nDATA BUFFER MISSING\n')
            sys.exit(1)

        to_return = self.data
        self.data = None

        return to_return

    def del_data(self):

        self.data = None

    #######################################################################

    def set_buffer(self, key, value):

        if self.data is not None:
            self.data[key] = value

    def get_buffer(self, key):

        if self.data is None:
            value = None
        else:
            value = self.data.get(key, None)

        return value

    def del_buffer(self, key):

        if self.data is not None:
            if key in self.data:
                del self.data[key]
