#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Manages the database connection and interaction.
"""

###########################################################################
# IMPORTS

import sys

from django.db import connection, transaction

#from MySQLdb.converters import conversions
#from MySQLdb.constants import FIELD_TYPE
#import MySQLdb.cursors as cursors

###########################################################################
# GLOBALS

#MY_CONV = { FIELD_TYPE.LONG: int, }
#conversions.update(MY_CONV)

###########################################################################
# CLASS

class DbCnx(object):
    """
    Manages the database connection and interaction.

    .. note:: It passes queries to the database, but it does not own the queries content.
    """

    #######################################################################
    # main functionality

    def __init__(self):
        """
        Initialize the object.
        """

        self.cursor = None

    def connect(self):
        """
        Connect to the database.
        """

        #self.cursor = connection.cursor()
        raise NotImplementedError

    def disconnect(self):
        """
        Disconnect from the database.
        """

        #self.cursor = None
        raise NotImplementedError

    #######################################################################

    def execute(self, query):
        """
        Pass the query to the database.
        """

        self.cursor = connection.cursor()
        self.cursor.execute(query)

    def commit(self):
        """
        Commit the stack of queries to the database.
        """

        transaction.commit_unless_managed()

    def fetchone(self):
        """
        Fetch the results from the database, one by one.
        """

        return self.cursor.fetchone()

    def fetchall(self):
        """
        Fetch the results from the database, all at once.
        """

        return self.cursor.fetchall()

    #######################################################################

    def custom_query(self, query):
        """
        Execute and commit an atomic query.

        Supported operations:
            - SELECT and fetch all data;
            - INSERT and return the ID;
            - UPDATE and return None;
            - CREATE and return None;
            - DROP and return None;
            - ALTER TABLE and return None;
            - RENAME TABLE and return None;
        """

        cursor = connection.cursor()

        answer = None
        if query.startswith('SELECT'):
            # data retrieval operation - no commit required
            cursor.execute(query)
            answer = cursor.fetchall()

        elif query.startswith('INSERT'):
            cursor.execute(query)
            transaction.commit_unless_managed()

            # retrieve the dbid
            cursor.execute("SELECT LAST_INSERT_ID()")
            answer = cursor.fetchone()[0]

        elif query.startswith('UPDATE'):
            cursor.execute(query)
            transaction.commit_unless_managed()

        elif query.startswith('CREATE'):
            cursor.execute(query)
            transaction.commit_unless_managed()

        elif query.startswith('DROP'):
            cursor.execute(query)
            transaction.commit_unless_managed()

        elif query.startswith('ALTER'):
            cursor.execute(query)
            transaction.commit_unless_managed()

        elif query.startswith('RENAME'):
            cursor.execute(query)
            transaction.commit_unless_managed()

        else:
            sys.stderr.write('\nUNKNOWN QUERY %s\n' % query)
            sys.exit(1)

        return answer
