#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Cython declarations
"""

###########################################################################
# IMPORTS

cimport numpy as cnp

from retrieval_libs.dblayer.DbTable cimport DbTable

###########################################################################
# CLASS

cdef class DbTableImg(DbTable):

    #######################################################################
    # CLASS variables

    cdef str DIR_DIST
    cdef str FFBUFFER_ID
    cdef str DDBUFFER_ID
    cdef list FFBUFFER
    cdef list DDBUFFER

    cdef cnp.ndarray all_hdbid
    cdef bint has_featdata

    #######################################################################
    # buffer management

    cdef _init_buffers(self)
    cdef set_buffers(self)
    cdef trim_buffers(self)

    #######################################################################
    # image features

    cdef tuple _ifeatdata(self, cnp.ndarray dbid, cnp.int32_t tt)
    cpdef tuple featdata(self, cnp.ndarray dbid, cnp.int32_t tt, bint buffered)

    #######################################################################
    # image distances

    cdef tuple distdata_feat(self, tuple kfeat, tuple hfeat)

    cdef tuple _distdata_dist(self,
            cnp.ndarray[cnp.int32_t, ndim=1, mode='c'] kdbid,
            cnp.ndarray[cnp.int32_t, ndim=1, mode='c'] hdbid,
            cnp.int32_t tt)

    cpdef tuple distdata(self,
            cnp.ndarray[cnp.int32_t, ndim=1, mode='c'] kdbid,
            cnp.ndarray[cnp.int32_t, ndim=1, mode='c'] hdbid,
            cnp.int32_t tt, bint buffered, bint special)

    cdef expcol_featdata(self,
            cnp.ndarray newdbid,
            cnp.ndarray dbid,
            list trix,
            cnp.ndarray deltadbid)

    cdef expcol_dd(self,
            cnp.ndarray seeddbid,
            cnp.ndarray newdbid,
            cnp.ndarray dbid,
            list trix,
            cnp.ndarray deltadbid)

###########################################################################
# CLASS

cdef class DbTableImgSynthetic(DbTableImg):

    #######################################################################
    # CLASS variables

    cdef str DIR_IMG

    #######################################################################
    # main functionality

    pass
