#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Updates the urls validity flags.
"""

###########################################################################

import logging
import os
import sys
import urllib2

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

BLOCKLN = 1000
SUBBLOCKLN = 100

###########################################################################

def _i_block(num, BLOCKLN):
    """
    Get the (indexes of the) block for processing.
    """

    if os.getenv('SGE_TASK_ID') is None:
        # data is processed in one block
        i_from = 1
        i_to = num + 1
    else:
        # data is processed in blocks, one block per job
        nb_blocks = num / BLOCKLN + 1

        job_id = int(os.getenv('SGE_TASK_ID'))
        job_index = job_id - 1

        i_from = min(job_index * BLOCKLN + 1, num + 1)
        i_to = min(job_index * BLOCKLN + BLOCKLN, num + 1)

        stepsize = int(os.getenv('SGE_TASK_STEPSIZE'))
        if stepsize != 1:
            sys.stderr.write('\nSGE_TASK_STEPSIZE %s != 1\n' % stepsize)
            sys.exit(1)

    return i_from, i_to

def check_url(imurl):
    """
    Check if the url is valid.
    """

    valid = True
    imname_request = None
    imname_response = None
    try:
        request = urllib2.Request(imurl)
        request.get_method = lambda : 'HEAD'

        response = urllib2.urlopen(request)
        answer = response.geturl()

        imname_request = imurl.split('/')[-1]
        imname_response = answer.split('/')[-1]
        if imname_request != imname_response:
            valid = False
    except:
        valid = False

    if not valid:
        logger.debug('VALID %s IMNAME %s != %s' %
                     (valid, imname_request, imname_response))

    return valid

def update_valid_url(dbimg):
    """
    Update the urls validity flags.
    """

    num = dbimg.size()

    i_from, i_to = _i_block(num, BLOCKLN)

    # process the data in SUBBLOCKS
    nb_ii = (i_to - i_from) / SUBBLOCKLN + 1
    for ii in xrange(0, nb_ii, 1):
        ii_from = i_from + ii * SUBBLOCKLN
        ii_to = min(ii_from + SUBBLOCKLN, i_to)

        if ii_from >= ii_to:
            continue

        logger.info('    PROGRESS INFO %s - %s' % (ii_from, ii_to))

        # get the subblock for processing
        dbid = range(ii_from, ii_to, 1)

        imurls = dbimg.im_path(dbid)
        for imdbid, imurl in imurls:

            valid = check_url(imurl)
            dbimg.update_valid_url(dbid, valid)

            logger.debug('DBID %s VALID %s URL %s' % (imdbid, valid, imurl))
