#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Provides the statistics of the image collection.
"""

############################################################

import logging

from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.figure import Figure

from numpy import int32, float32, inf, array, zeros, arange, cumsum, histogram, log
import operator
import os
import pylab as P

#from math import log
from random import Random
#import statistics
import string
import sys
from time import time as now

from retrieval_libs.dblayer.FileAccess import read_bindata

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')
logger.setLevel(logging.INFO)

random = Random()
random.jumpahead(8)

FLAG_LOG_SCALE = True

DPI = 600

###########################################################################
# CLASS

class DbImg_Statistics(object):
    """
    Provides the statistics of the image collection.
    """

    #######################################################################
    # local functionality

    def _dbid(self, nb_samples):
        """
        Return a random set of image ids.
        """

        if nb_samples is None:
            dbid = self.dbimg.all_ids()

        elif nb_samples == self.dbidln:
            dbid = self.dbimg.all_ids()

        elif nb_samples < self.dbidln:
            dbid = []
            while len(dbid) < nb_samples:
                idbid = int32(random.randint(1, self.dbidln))
                if idbid not in dbid:
                    dbid.append(idbid)

            dbid = array(dbid, dtype=int32, order='c')

        else:
            pass

        return dbid

    #######################################################################
    #  main functionality

    def init(self, dbaccess, thargs, weights):
        """
        Initialize the object.
        """

        self.dbimg = dbaccess.dbimg()
        self.dbfig = dbaccess.dbfig()

        self.thargs = thargs
        self.weights = weights

        self.dbidln = self.dbimg.size()

    #######################################################################

    def _hist(self, kdbid, hdbid, tt):
        """
        """

        if len(kdbid) != 1:
            sys.stderr.write('\nERROR len(kdbid) should be 1')
            sys.exit(1)

        #kdbid[0] = 30307
        distdata = self.dbimg.distdata(kdbid, hdbid, tt, False, False)
        x = distdata[0]
        print 'x', x.shape#, ' '.join(map(str, x))
        xmax = 2000

        count, bins = histogram(x, bins=arange(0, xmax, xmax/200.0), normed=False)

        # allign
        bins = bins[:-1]

        # log-scale
        if FLAG_LOG_SCALE:
            count = log(count)
            for i in xrange(0, len(bins), 1):
                if count[i] == -inf:
                    count[i] = 0

        # normalize
        count = count / float(sum(count))

        #hdbidln = float(len(hdbid))
        #count = [value / hdbidln for value in count]
        #print '_hist', len(bins), len(x), sum(n)

        return bins, count

    def _hist2(self, kdbid, hdbid, tt, oldt='', newt=''):
        """
        """

        if len(kdbid) != 1:
            sys.stderr.write('\nERROR len(kdbid) should be 1')
            sys.exit(1)

        #kdbid[0] = 10
        #distdata = self.dbimg.distdata(kdbid, hdbid, tt, False, False)
        distfile = self.dbimg.distfile(kdbid)[0][1]

        if oldt != newt:
            distfile = distfile.replace(oldt, newt)

        distdata = read_bindata(distfile)

        x = distdata[:,tt]
        print 'x', x.shape#, ' '.join(map(str, x))
        xmax = 2000

        count, bins = histogram(x, bins=arange(0, xmax, xmax/200.0), normed=False)

        # allign
        bins = bins[:-1]

        # log-scale
        if FLAG_LOG_SCALE:
            count = log(count)
            for i in xrange(0, len(bins), 1):
                if count[i] == -inf:
                    count[i] = 0

        # normalize
        count = count / float(sum(count))

        #hdbidln = float(len(hdbid))
        #count = [value / hdbidln for value in count]
        #print '_hist', len(bins), len(x), sum(n)

        return bins, count

    def histogram(self, tt):
        """
        Plot the histogram of the image distances.
        """

        fig = Figure(figsize=(4, 4), dpi=DPI)
        ax = fig.add_subplot(111)

        texts = []
        ll = []

        h_samples = self.dbidln#1000#
        #hdbid = self._dbid(h_samples)

        k_samples = 100

        kdbid2 = []

        bins = None
        ytmp = None
        y = None
        for index in xrange(0, k_samples, 1):

            print 'Progress info: index ', index

            kdbid = self._dbid(1)
            kdbid2.append(kdbid)
            hdbid = self._dbid(h_samples)

            #bins, ytmp = self._hist(kdbid, hdbid, tt)
            bins, ytmp = self._hist2(kdbid, hdbid, tt, oldt='', newt='')

            #print 'local', sum(ytmp[:10])
            # cumulative
            #ytmp = list(cumsum(ytmp))

            # histogram
            #line = ax.step(bins, ytmp, linewidth=1, color='b')
            #ll.append(line[0])

            if y is None:
                y = ytmp[:]
            else:
                for i in xrange(0, len(y), 1):
                    y[i] += ytmp[i]

        k_samples = float(k_samples)
        y = [value / k_samples for value in y]

        # average histogram
        line = ax.step(bins, y, linewidth=2, color='r')
        texts.append('iso')
        ll.append(line[0])







        bins = None
        ytmp = None
        y = None
        for index, kdbid in enumerate(kdbid2):

            print 'Progress info: index ', index

            hdbid = self._dbid(h_samples)

            #bins, ytmp = self._hist2(kdbid, hdbid, tt, oldt='_iso', newt='')
            bins, ytmp = self._hist2(kdbid, hdbid, tt, oldt='distances_biased', newt='distances')

            #print 'local', sum(ytmp[:10])
            # cumulative
            #ytmp = list(cumsum(ytmp))

            # histogram
            #line = ax.step(bins, ytmp, linewidth=1, color='g')
            #ll.append(line[0])

            if y is None:
                y = ytmp[:]
            else:
                for i in xrange(0, len(y), 1):
                    y[i] += ytmp[i]

        k_samples = float(k_samples)
        y = [value / k_samples for value in y]

        # average histogram
        line = ax.step(bins, y, linewidth=2, color='b')
        texts.append('ecl')
        ll.append(line[0])




        #ax.set_xlabel('distance')
        #ax.set_ylabel('cumulative pdf')
        #ax.set_title(r'$\mathrm{Histogram}$')
        #ax.set_title('%s' % kdbid)
        #ax.axis([0.0, 1.1, 0.0, 1.1])
        #ax.grid(True)

        if len(ll) > 0:
            ax.legend(ll, texts,'upper right')

        filename, fileurl = self.dbfig.filename('histograms', 'hist', '.png')
        canvas = FigureCanvas(fig)
        canvas.print_figure(filename, dpi=DPI)

        return





        bins = None
        ytmp = None
        y = None
        for index, kdbid in enumerate(kdbid2):

            print 'Progress info: index ', index

            hdbid = self._dbid(h_samples)

            bins, ytmp = self._hist2(kdbid, hdbid, tt, oldt='iso', newt='mht')

            #print 'local', sum(ytmp[:10])
            # cumulative
            #ytmp = list(cumsum(ytmp))

            # histogram
            line = ax.step(bins, ytmp, linewidth=1, color='y')
            texts.append('mth')
            ll.append(line[0])

            if y is None:
                y = ytmp[:]
            else:
                for i in xrange(0, len(y), 1):
                    y[i] += ytmp[i]

        k_samples = float(k_samples)
        y = [value / k_samples for value in y]

        # average histogram
#        ax.step(bins, y, linewidth=3, color='y')

        bins = None
        ytmp = None
        y = None
        for index, kdbid in enumerate(kdbid2):

            print 'Progress info: index ', index

            hdbid = self._dbid(h_samples)

            bins, ytmp = self._hist2(kdbid, hdbid, tt, oldt='iso', newt='fra')

            #print 'local', sum(ytmp[:10])
            # cumulative
            #ytmp = list(cumsum(ytmp))

            # histogram
            line = ax.step(bins, ytmp, linewidth=1, color='k')
            texts.append('fra')
            ll.append(line[0])

            if y is None:
                y = ytmp[:]
            else:
                for i in xrange(0, len(y), 1):
                    y[i] += ytmp[i]

        k_samples = float(k_samples)
        y = [value / k_samples for value in y]

        # average histogram
#        ax.step(bins, y, linewidth=3, color='y')

    #######################################################################

    def _hist_average(self, kdbid, hdbid, tt):
        """
        Return the histogram of the image distances.
        """

        fig = Figure(figsize=(4, 4), dpi=DPI)
        ax = fig.add_subplot(111)

        distdata = self.dbimg.featdens(hdbid, tt)
        x = distdata[0]
        xmax = x.max()
        print 'xmax', xmax

        n, bins, patches = ax.hist(x, bins=arange(0, xmax, 0.1), normed=False)

        # normalize
        hdbidln = float(len(hdbid))
        n = [value / hdbidln for value in n]
        # allign
        bins = bins[:-1]
        #print '_hist', len(bins), len(x), sum(n)

        return bins, n

    def histogram_average(self, tt):
        """
        Return the us
        """

        fig = Figure(figsize=(4, 4), dpi=DPI)
        ax = fig.add_subplot(111)

        h_samples = self.dbidln
        k_samples = 1

        bins = None
        ytmp = None
        y = None
        for index in xrange(0, k_samples, 1):

            print 'Progress info: index ', index

            kdbid = self._dbid(1)
            hdbid = self._dbid(h_samples)

            bins, ytmp = self._hist_average(kdbid, hdbid, tt)

            #for i in xrange(1, len(bins), 1):
            #    bins[i] = log(bins[i])
            #ytmp = list(cumsum(ytmp))

            # cumulative histogram
            ax.step(bins, ytmp, linewidth=1, color='b')

            if y is None:
                y = ytmp[:]
            else:
                for i in xrange(0, len(y), 1):
                    y[i] += ytmp[i]

        k_samples = float(k_samples)
        y = [value / k_samples for value in y]

        # average cumulative histogram
        ax.step(bins, y, linewidth=3, color='r')

        ax.set_xlabel('Average distance to the closest 100 images')
        ax.set_ylabel('Cumulative pdf')
        ax.set_title(r'$\mathrm{ImageNet.33K}$')
        #ax.axis([0.0, 1.1, 0.0, 1.1])
        #ax.grid(True)

        filename, fileurl = self.dbfig.filename('histograms', 'ddavg', '.png')
        canvas = FigureCanvas(fig)
        canvas.print_figure(filename, dpi=DPI)

    #######################################################################

    def _average(self, kdbid, hdbid, tt, percentage=10.0):
        """
        Returns the average distance for the given image to
        the given percentage of the entire collection.
        """

        if len(kdbid) != 1:
            sys.stderr.write('\nERROR len(kdbid) should be 1')
            sys.exit(1)

        h_samples = len(hdbid)
        index_ref = int(h_samples * percentage / 100.0)

        distdata = self.dbimg.distdata(kdbid, hdbid, tt, False, False)
        distlist = distdata[0]

        # order the distances, in increasing order
        distlist.sort(axis=0)

        dcurrent = sum(distlist[0:index_ref]) / index_ref

        logger.info(
                'kdbid %s, h_samples %s index_ref %s dcurrent %s'
                % (kdbid, h_samples, index_ref, dcurrent))

        return dcurrent

    #######################################################################

    def _threshold(self, kdbid, hdbid, tt, percentage=10.0):
        """
        Returns the similarity threshold for the given image that
        includes the given percentage of the entire collection.
        """

        if len(kdbid) != 1:
            sys.stderr.write('\nERROR len(kdbid) should be 1')
            sys.exit(1)

        h_samples = len(hdbid)
        index_ref = int(h_samples * percentage / 100.0)

        distdata = self.dbimg.distdata(kdbid, hdbid, tt, False, False)
        distlist = distdata[0]

        # order the distances, in increasing order
        distlist.sort(axis=0)

        dcurrent = distlist[index_ref]

        logger.info(
                'kdbid %s, h_samples %s index_ref %s dcurrent %s'
                % (kdbid, h_samples, index_ref, dcurrent))

        return dcurrent

    def threshold(self, tt, percentage=10.0, imax=None):
        """
        Returns the similarity threshold for all images in overall that
        includes the given percentage of the entire collection.

        TODO: it works properly only for tt in (0, 1)
        """

        h_samples = self.dbidln#1000
        hdbid = self._dbid(h_samples)

        if imax is None:
            k_samples = self.dbidln#1000
        else:
            k_samples = imax

        index_ref = int(h_samples * percentage / 100.0)

        dtotal = 0.0
        dcount = 0
        daverage = 0.0
        for index in xrange(0, k_samples, 1):

            kdbid = self._dbid(1)

            distdata = self.dbimg.distdata(kdbid, hdbid, tt, False, False)
            distlist = distdata[0]

            # order the distances, in increasing order
            distlist.sort(axis=0)

            dcurrent = distlist[index_ref,0]
            if dcurrent >= 1000000.0:
                continue

            dtotal += dcurrent
            dcount += 1

            # update the estimation
            daverage = dtotal / dcount

            logger.info('index %s dcurrent %s daverage_estimation %s' %
                        (index, dcurrent, daverage))

        return daverage

    #######################################################################

    def _vecinity(self, kdbid, hdbid, tt):
        """
        """

        if len(kdbid) != 1:
            sys.stderr.write('\nERROR len(kdbid) should be 1')
            sys.exit(1)

        distdata = self.dbimg.distdata(kdbid, hdbid, tt, False, False)
        distlist = distdata[0]

        min_dist = distlist.max()
        hdbid_ln = hdbid.shape[0]
        for i in xrange(0, hdbid_ln, 1):

            # skip the image itself
            if hdbid[i] == kdbid[0]:
                continue

            if distlist[i] < min_dist:
                min_dist = distlist[i]

        logger.info('kdbid %s minimum distance %s' % (kdbid, min_dist))

        return min_dist

    def vecinity(self, tt):
        """
        Plot the histogram of the distance to the closest image.
        """

        h_samples = self.dbidln#1000
        hdbid = self._dbid(h_samples)

        k_samples = 100#self.dbidln#1000

        distances = []
        for index in xrange(0, k_samples, 1):

            kdbid = self._dbid(1)

            dist_min = self._vecinity(kdbid, hdbid, tt)
            distances.append(dist_min)

        ###
        fig = Figure(figsize=(4, 4), dpi=DPI)
        ax = fig.add_subplot(111)

        xmax = max(distances) * 1.1

        n, bins = histogram(distances, bins=arange(0, xmax, xmax/100.0), normed=False)

        # normalize
        hdbidln = float(len(hdbid))
        n = [value / hdbidln for value in n]
        # allign
        bins = bins[:-1]
        #print '_hist', len(bins), len(x), sum(n)

        # average histogram
        ax.step(bins, n, linewidth=3, color='r')

        ax.set_xlabel('Average distance to the closest image')
        #ax.set_ylabel('Cumulative pdf')
        #ax.set_title(r'$\mathrm{ImageNet.33K}$')
        #ax.axis([0.0, 1.1, 0.0, 1.1])
        #ax.grid(True)

        filename, fileurl = self.dbfig.filename('histograms', 'ddavg', '.png')
        canvas = FigureCanvas(fig)
        canvas.print_figure(filename, dpi=DPI)

    #######################################################################

    def _nb_within(self, kdbid, hdbid, tt):
        """
        """

        if len(kdbid) != 1:
            sys.stderr.write('\nERROR len(kdbid) should be 1')
            sys.exit(1)

        distdata = self.dbimg.distdata(kdbid, hdbid, tt, False, False)
        dd0 = distdata[0]

        thpX0 = self.thargs[0][0,tt]
        #print 'thpX0', self.thargs, thpX0

        counter = 0
        for i in xrange(0, dd0.shape[0], 1):
            if dd0[i] < thpX0:
                counter += 1

        logger.info('kdbid %s counter %s' % (kdbid, counter))

        return counter

    def nb_within(self, tt):
        """
        Count the images within the similarity range/threshold.
        """

        h_samples = self.dbidln#1000
        hdbid = self._dbid(h_samples)

        k_samples = self.dbidln#1000

        for index in xrange(0, k_samples, 1):

            kdbid = self._dbid(1)

            counter = self._nb_within(kdbid, hdbid, tt)

            logger.info('index %s counter %s' % (index, counter))


    def check_symm(self):
        """
        """

        tt = 0
        #kdbid = self._dbid(1)
        while True:
            kdbid = self._dbid(1)
            hdbid = self._dbid(1)

            dd_from_k = self.dbimg.distdata(kdbid, hdbid, tt, False, False)
            #if dd_from_k[0][0,0] >= 1000.0:
            #    continue

            dd_from_h = self.dbimg.distdata(hdbid, kdbid, tt, False, False)

            #if dd_from_k[0][0,0] == dd_from_h[0][0,0]:
            print 'dd', dd_from_k[0][0,0], dd_from_h[0][0,0]
