#! /usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Creates the tables for the Task, Config, Session models.
"""

###########################################################################
# IMPORTS

import logging

from retrieval_libs.dblayer.DbAccess import DbAccess

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

# implemented as a singleton
dbaccess = DbAccess()

###########################################################################

def init_config_table(TABLE_CONFIG):
    """
    Initialize the Config model table.
    """


    #dbaccess.custom_query("DROP TABLE %s" % TABLE_CONFIG)
    dbaccess.custom_query(
            "CREATE TABLE %s ("
            "id INTEGER NOT NULL auto_increment, "
            "name VARCHAR(100), "
            "TRACE VARCHAR(100) NOT NULL, "
            "SYS_THARGS VARCHAR(100) NOT NULL, "
            "SYS_WEIGHTS VARCHAR(100) NOT NULL, "
            "SYS_CONSISTENCY VARCHAR(100) NOT NULL, "
            "SYS_VERSION INTEGER NOT NULL, "
            "datetime DATETIME NOT NULL, "
            "PRIMARY KEY (id)"
            ")"
            % TABLE_CONFIG)

def init_task_table(TABLE_TASK):
    """
    Initialize the Task model table.
    """

    #dbaccess.custom_query("DROP TABLE %s" % TABLE_TASK)
    dbaccess.custom_query(
            "CREATE TABLE %s ("
            "id INTEGER NOT NULL auto_increment, "
            "query VARCHAR(100) NOT NULL, "
            "target LONGTEXT NOT NULL, "
            "USR_THARGS VARCHAR(100) NOT NULL, "
            "USR_WEIGHTS VARCHAR(100) NOT NULL, "
            "USR_VERSION VARCHAR(100) NOT NULL, "
            "datetime DATETIME NOT NULL, "
            "PRIMARY KEY (id)"
            ")"
            % TABLE_TASK)

def init_session_table(TABLE_SEARCH, TABLE_USER, TABLE_TASK, TABLE_CONFIG):
    """
    Initialize the Session model table.
    """

    #dbaccess.custom_query("DROP TABLE %s" % TABLE_SEARCH)
    dbaccess.custom_query(
            "CREATE TABLE %s ("
            "id INTEGER NOT NULL auto_increment, "
            "user_id INTEGER NOT NULL, "
            "task_id INTEGER NOT NULL, "
            "config_id INTEGER NOT NULL, "
            "iterations INTEGER NOT NULL, "
            "evaluation VARCHAR(1) NOT NULL, "
            "action VARCHAR(1000) NOT NULL, "
            "thargs VARCHAR(5000) NOT NULL, "
            "weights VARCHAR(1000) NOT NULL, "
            "consistency VARCHAR(1000) NOT NULL, "
            "dispfdbk VARCHAR(5000) NOT NULL, "
            "trace LONGTEXT, "
            "timing VARCHAR(5000) NOT NULL, "
            "datetime DATETIME NOT NULL, "
            "PRIMARY KEY (id), "
            "FOREIGN KEY (user_id) REFERENCES %s(id) ON DELETE CASCADE ON UPDATE CASCADE, "
            "FOREIGN KEY (task_id) REFERENCES %s(id) ON DELETE CASCADE ON UPDATE CASCADE, "
            "FOREIGN KEY (config_id) REFERENCES %s(id) ON DELETE CASCADE ON UPDATE CASCADE"
            ")"
            % (TABLE_SEARCH, TABLE_USER, TABLE_TASK, TABLE_CONFIG))
