#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Provides the display information for the retrieved results.
"""

###########################################################################
# IMPORTS

import logging
import sys
from time import time as now

from retrieval_libs.dblayer.DbAccess import DbAccess

###########################################################################
# GLOBALS

logger = logging.getLogger('retrieval')

# implemented as a singleton
dbaccess = DbAccess()

###########################################################################

def _im_css_style(im_fdbk):

    if im_fdbk == 9:
        # REFERENCE IMAGE - very dark red
        style = "border: 4px #330000 solid;"
    elif im_fdbk == 10:
        # TARGET CLASS - very dark red
        style = "border: 4px #660000 solid;"
    elif im_fdbk == 0:
        # ORDERED IMAGES -dark red
        style = "border: 4px #800000 solid;"
    elif im_fdbk == 1:
        # ORDERED IMAGES -red
        style = "border: 4px #D50000 solid;"
    elif im_fdbk == 2:
        # ORDERED IMAGES - light red
        style = "border: 4px #FF0B0B solid;"
    elif im_fdbk == 3:
        # ORDERED IMAGES - very light red
        style = "border: 4px #FF5555 solid;"
    elif im_fdbk == 4:
        # ORDERED IMAGES -red
        style = "border: 4px #FF9595 solid;"
    elif im_fdbk == 5:
        # ORDERED IMAGES - light red
        style = "border: 4px #FFBFBF solid;"
    elif im_fdbk == 6:
        # ORDERED IMAGES - very light red
        style = "border: 4px #FFDFDF solid;"
    elif im_fdbk == 7:
        # ORDERED IMAGES - very light red
        style = "border: 4px #FFEAEA solid;"
    else:
        # OTHERS - none
        style = "border: 0px;"

    return style

###########################################################################
# CLASS

class AnswerMain(object):
    """
    Provides the display information for the retrieved results.
    """

    #######################################################################
    #  local functionality

    def _prepare_answer(self, dispdata):

        imdbid = dispdata.get('imdbid', None)
        if imdbid is None:
            return

        self.answers = []

        # FORM: ((dbid, pathToOrig),...)
        iminfo = self.dbimg.info(imdbid)

        # default style
        imstyle = _im_css_style (None)

        for index, item in enumerate(iminfo):

            imdbid, impath = item

            # the information necessary for displaying one image
            item = {}
            item['index'] = index + 1

            image = {'index': 0,
                     'imdbid': imdbid,
                     'impath': impath,
                     'imstyle': imstyle,
                     'distToRefIm': None}

            item['images'] = [image]
            self.answers.append(item)

    def _prepare_extra(self, dispdata):

        dispprob = dispdata.get('improb', None)
        if dispprob is None:
            return

        for index, item in enumerate(self.answers):

             for iminfo in item['images']:
                iminfo['improb'] = dispprob[index]

    def _prepare_assist(self, assistinfo):

        if self.answers:

            for index, item in enumerate(self.answers):

                for iminfo in item['images']:
                    imdbid = iminfo['imdbid']
                    imorder = assistinfo.get(imdbid, None)
                    imstyle = _im_css_style(imorder)

                    #iminfo['imorder'] = imorder
                    iminfo['imstyle'] = imstyle

    #######################################################################
    # main functionality

    def init(self):
        """
        Initialize the object.
        """

        self.dbimg = dbaccess.dbimg()

    def display_info(self, dispdata, assistinfo=None):
        """
        Prepare the display information.

        Keyword arguments:
        dispdata   --
        assistinfo -- list of additional information corresponding to dispdata,
                      which is provided by the assistant application
        """

        #logger.debug('START')
        #start_time = now()

        if dispdata is None:
            self.answers = None

        else:
            self._prepare_answer(dispdata)
            self._prepare_extra(dispdata)
            if assistinfo is not None:
                self._prepare_assist(assistinfo)

        #stop_time = now()
        #logger.debug('DONE in %f seconds' % (stop_time - start_time))

        return self.answers
