#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Provides the plots of the system performance.
"""

###########################################################################
# IMPORTS

from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.figure import Figure
#from matplotlib.table import Table

from numpy import float32, array, zeros, arange
from pylab import table, hist

from django.conf import settings

###########################################################################
# GLOBALS

DEFAULT_DPI = 600#settings.DPI_STATGFX

# global variables
#LABELSLONG
#LABELS
#COLORS
#WIDTHS
#MARKERS

###########################################################################
# FUNCTIONS

def init(config_props):

    global LABELSLONG
    global LABELS
    global COLORS
    global WIDTHS
    global MARKERS

    LABELSLONG = config_props['LABELSLONG']
    LABELS = config_props['LABELS']
    COLORS = config_props['COLORS']
    WIDTHS = config_props['WIDTHS']
    MARKERS = config_props['MARKERS']

###########################################################################
# FUNCTIONS

def figure_0(configs, yinfo, tinfo, utisum, utsum, histo, filename):

    fig = Figure(figsize=(8, 6), dpi=DEFAULT_DPI)
    ax = fig.add_subplot(111)

    bins = arange(0, histo.shape[1], 1)
    texts = []
    ll = []
    colours = []
    for cid, c in configs.iteritems():

        if utisum[c] < 1:
            continue

        color = COLORS[cid]
        width = WIDTHS[cid]
        marker = MARKERS[cid]
        name = LABELSLONG[cid]
        #text = 'uti%d, ut%d, %s' % (utisum[c], utsum[c], name)
        #text = 'uti%d, %s' % (utisum[c], name)
        text = '%s' % (name)
        texts.append(text)

        y = histo[c,:]

        line = ax.step(bins, y, where='post', linewidth=width, marker=marker, color=color)
        ll.append(line[0])

    ax.set_xlabel('Number of iterations')
    xbins = array([0.0, 5.0, 10.0, 15.0, 20.0])
    ax.set_xticks(xbins+0.5)
    ax.set_xticklabels(['0','5','10','15','20'], ha='center', fontdict=6)
    ax.set_xlim(0.0, 21.0)
    ax.set_ylabel(yinfo)
    ax.set_ylim(0.0, 1.0)
#    ax.set_title(tinfo)
    ax.grid(True)
    if len(ll) > 0:
        ax.legend(ll, texts,'upper left')
        #ax.legend(ll, texts,'lower right')

    canvas = FigureCanvas(fig)
    canvas.print_figure(filename)

###########################################################################

def figure_eval_fdbk(username, configs, yinfo, tinfo, utisum, histo, filename):

    fig = Figure(figsize=(4, 4), dpi=DEFAULT_DPI)
    ax = fig.add_subplot(111)

    LABELS = [username, 'all users']
    COLORS = ['r','k']
    WIDTHS = ['3','1']

    bins = arange(0, histo.shape[1], 1)
    texts = []
    ll = []
    for cid, c in configs.iteritems():

        if utisum[c] < 1:
            continue

        y = histo[c,:]

        color = COLORS[cid]
        width = WIDTHS[cid]
        marker = MARKERS[cid]
        name = LABELS[cid]
        text = '%s' % (name)
        texts.append(text)

        line = ax.step(bins, y, where='post', linewidth=width, marker=marker, color=color)
        ll.append(line[0])

    ax.set_xlabel('Number of iterations')
    xbins = array([0.0, 5.0, 10.0, 15.0, 20.0])
    ax.set_xticks(xbins+0.5)
    ax.set_xticklabels(['0','5','10','15','20'], ha='center', fontdict=6)
    ax.set_xlim(0.0, 21.0)
    ax.set_ylabel(yinfo)
    ax.set_ylim(0.0, 1.0)
    ax.set_title(tinfo)
    ax.grid(True)
    if len(ll) > 0:
        ax.legend(ll, texts,'lower right')

    canvas = FigureCanvas(fig)
    canvas.print_figure(filename, dpi=DEFAULT_DPI)

###########################################################################

def figure_12(configs, yinfo, tinfo, data, filename):

    fig = Figure(figsize=(8, 6), dpi=DEFAULT_DPI)
    ax = fig.add_subplot(111)

    binedges = array([x + 0.5 for x in xrange(-1, 21, 1)])
    bincenters = 0.5 *(binedges[1:] + binedges[:-1])
    width = 0.25
    bins = arange(0, data.shape[3], 1)
    texts = []
    ll = []
    for cid, c in configs.iteritems():

        #if utisum[c] < 2:
        #    continue

        color = COLORS[cid]
        name = LABELSLONG[cid]
        text = '%s' % (name)
        texts.append(text)

        shape = data.shape
        #print 'shape', shape
        #y = zeros((21, shape[0] + shape[1]), dtype=float32, order='c')
        #for i in xrange(0, shape[2], 1):
        for u in xrange(0, shape[0], 1):
            for t in xrange(0, shape[1], 1):
                #y[i,(u+t)] = data[u,t,c,i]

        #print 'y', y

        #for ut in xrange(0, (shape[0] + shape[1]), 1):
            #line = ax.step(bincenters, y, where='post', linewidth=1, color=color)
            #line = ax.bar(bincenters+0.5+(c-1.5)*width, y, width, color=color)
            #line = ax.plot(bincenters+0.5+(c-1.5)*width, data[u,t,c,:-1], 'o', linewidth=1, markersize=6, color=color)
                line = ax.plot(bincenters+0.5+(c-0.25)*width, data[u,t,c,:-1], 'o', linewidth=1, markersize=6, color=color)

                ll.append(line[0])

    ax.set_xlabel('Number of iterations')
    xbins = array([0.0, 5.0, 10.0, 15.0, 20.0])
    ax.set_xticks(xbins+0.5)
    ax.set_xticklabels(['0','5','10','15','20'], ha='center', fontdict=6)
    ax.set_xlim(0.0, 21.0)
    ax.set_ylabel(yinfo)
    ax.set_ylim(0.0, 6.0)
    ax.set_title(tinfo)
    ax.grid(True)
    if len(ll) > 0:
        ax.legend(ll[::-1], texts[::-1],'upper right')

    canvas = FigureCanvas(fig)
    canvas.print_figure(filename, dpi=DEFAULT_DPI)

###########################################################################

def figure_15(configs, yinfo, tinfo, data, filename):

    fig = Figure(figsize=(8,4), dpi=DEFAULT_DPI)
    ax = fig.add_subplot(111)

    nb_c = 0
    for c in xrange(0, data.shape[0], 1):
        for i in xrange(0, data.shape[1]-1, 1):
            if data[c,i,0] >= 0:
                nb_c += 1
                break

    if nb_c == 0:
        return

    binedges = array([x + 0.5 for x in xrange(-1, 21, 1)])
    bincenters = 0.5 * (binedges[1:] + binedges[:-1])
    width = 0.25 / nb_c

    cc = 0
    texts = []
    ll = []
    for cid, c in configs.iteritems():

        #if data[c,0,0] < 0:
        #print data[c,:,0]
        if data[c,-2,0] < 0:
            continue

        color = COLORS[cid]
        name = LABELSLONG[cid]
        text = '%s' % (name)
        texts.append(text)

        #means = data[c,:-1,0]
        #stds = data[c,:-1,1]

        means = []
        stds = []
        ind = []
        for i in xrange(0, data.shape[1]-1, 1):
            if data[c,i,0] >= 0:
                means.append(data[c,i,0])
                stds.append(data[c,i,1])
                ind.append(bincenters[i])

        if len(means) > 0:
            ind = [x + (cc - (nb_c - 1) / 2.0) * width for x in ind]
            cc += 1
            line = ax.bar(ind, means, width=0, color='w', edgecolor='w', yerr=stds, ecolor=color, capsize=3)
            line2 = ax.plot(ind, means, 'o', linewidth=1, markersize=3, color=color)

            ll.append(line2[0])

    ax.set_xlabel('Number of iterations')
    xbins = array([0.0, 5.0, 10.0, 15.0, 20.0])
    ax.set_xticks(xbins)
    ax.set_xticklabels(['0','5','10','15','20'], ha='center', fontdict=6)
    ax.set_xlim(-0.5, 20.5)
    ax.set_ylabel(yinfo)
    ax.set_ylim(-0.2, 1.2)#ax.set_ylim(0.0, 1.0)
    ax.set_title(tinfo)
    ax.grid(True)
    #if len(ll) > 0:
    #    ax.legend(ll[::-1], texts[::-1],'upper left')

    canvas = FigureCanvas(fig)
    canvas.print_figure(filename, dpi=DEFAULT_DPI)

def figure_displayed_images(configs, tinfo, xinfo, yinfo, data, filename):

    fig = Figure(figsize=(8, 6), dpi=DEFAULT_DPI)
    ax = fig.add_subplot(111)

    #maxln = 20
    #for data_c in data.values():
    #    for x in data_c.iterkeys():
    #        if x > maxln:
    #            maxln = x

    texts = []
    ll = []
    for cid, c in configs.iteritems():

        data_c = data.get(c, {})
        if len(data_c) < 1:
            continue

        color = COLORS[cid]
        width = WIDTHS[cid]
        marker = MARKERS[cid]
        name = LABELSLONG[cid]
        text = '%s' % (name)
        texts.append(text)

        bins = data_c.keys()
        y = data_c.values()

        line = ax.step(bins, y, where='post', linewidth=width, marker=marker, color=color)
        ll.append(line[0])

    ax.set_xlabel(xinfo)
    xbins = array([0.0, 5.0, 10.0, 15.0, 20.0])
    ax.set_xticks(xbins)
    ax.set_xticklabels(['0','5','10','15','20'], ha='center', fontdict=6)
    #ax.set_xlim(0, maxln)
    ax.set_ylabel(yinfo)
    ax.set_ylim(0.0, 1.0)
    ax.set_title(tinfo)
    ax.grid(True)
    #if len(ll) > 0:
    #    ax.legend(ll, texts,'upper right')

    canvas = FigureCanvas(fig)
    canvas.print_figure(filename, dpi=DEFAULT_DPI)
