#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Provides methods to generate the synthetic images.
"""

###########################################################################
# IMPORTS

from numpy import array

from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
from matplotlib.figure import Figure

###########################################################################
# GLOBALS

DEFAULT_DPI = 80

#COLORS = ['#800000', '#D50000', '#FF0B0B', '#FF5555', '#FF9595', '#FFBFBF', '#FFDFDF', '#FFEAEA',  '#660000']
COLORS = ['r', 'g', 'b', 'c', 'm', 'y', '#afffaa', '#afeeee', '#660000']

###########################################################################
# FUNCTION

def generate_image2D(coord2D, filename):

    fig = Figure(figsize=(3, 3), dpi=DEFAULT_DPI)
    ax = fig.add_subplot(111)

    line = ax.plot(coord2D[0], coord2D[1], linewidth=3, marker='o', markersize=6, color='k')

    ax.set_xlabel('')
    xbins = array([0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0])
    ax.set_xticks(xbins)
    ax.set_xticklabels(['','','','','','','','','',''], ha='center', fontdict=6)
    ax.set_xlim(-0.01, 1.01)

    ax.set_ylabel('')
    ybins = array([0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0])
    ax.set_yticks(xbins)
    ax.set_yticklabels(['','','','','','','','','',''], ha='center', fontdict=6)
    ax.set_ylim(-0.01, 1.01)

    #ax.set_title(tinfo)
    ax.grid(True)

    canvas = FigureCanvas(fig)
    canvas.print_figure(filename, dpi=DEFAULT_DPI)
