#!/usr/bin/env python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Cython declarations
"""

###########################################################################
# IMPORTS

cimport numpy as cnp

from retrieval_libs.dblayer.DbTableImg cimport DbTableImg
from retrieval_libs.dblayer.DbTableTree cimport DbTableTree

###########################################################################
# CLASS

cdef class TraceUpdater(object):

    #######################################################################
    # CLASS variables

    cdef DbTableImg dbimg
    cdef DbTableTree dbtree

    cdef cnp.int32_t TRACE_LN

    cdef cnp.ndarray skipped_nodes

    #######################################################################
    # local functionality

    cdef dict _tracedata(self, cnp.ndarray tndbid, bint unlock)
    cdef dict _deltadata(self, cnp.ndarray tndbid)

    #######################################################################
    # local functionality

    cdef dict _expcol_rankdata(self, dict rankdata, list trix, dict deltadata)

    #######################################################################
    # local functionality

    cdef dict _update_rankdata(self, dict rankdata)

    #######################################################################
    # local functionality

    cdef dict _init_trace_recursive(self, dict trace, bint recursive)
    cdef cnp.ndarray _init_trace(self)

    #######################################################################
    #  main functionality

    cdef set_skipped_nodes(self, cnp.ndarray skipped)

    cdef init(self, DbTableImg dbimg, DbTableTree dbtree, tuple TRACE)
    cdef dict rankdata(self, dict rankdata)
    cdef dict clean_rankdata(self, dict rankdata)

    #######################################################################
    # expand/collapse functionality

    cdef _reformat_specs(self, cnp.int32_t dataln, cnp.ndarray trix_to_copy_from, cnp.int32_t deltaln)
    cdef _reformat_specs2(self,
            cnp.ndarray[cnp.int32_t, ndim=1, mode='c'] old_imdbid,
            cnp.ndarray[cnp.int32_t, ndim=1, mode='c'] new_imdbid)

    cdef tuple expand_specs(self, dict rankdata, node)
    cdef dict collapse_specs(self, dict rankmeta, dict rankdata_true)
    cdef dict init_collapse(self, dict rankdata)
    cdef tuple exit_collapse(self, dict rankmeta)
    cdef do_collapse(self, dict rankmeta, dict deltadata)

    cdef dict expcol_rankdata(self, dict rankdata, cnp.ndarray trix_to_copy_from, dict deltadata)
    cdef expcol_databuffer(self, cnp.ndarray seeddbid, cnp.ndarray newdbid, cnp.ndarray dbid)
