#!/usr/bin/env python

#cython: boundscheck=False
#cython: wraparound=False
#cython: embedsignature=True
#cython: profile=True

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Manages the rankdata.

.. note:: Rankdata has the size of the collection at hand.
"""

###########################################################################
# IMPORTS

import logging
from numpy import int32, float32, array, zeros, empty, ones
import sys
from time import time as now

from retrieval_libs.dblayer.DbAccess import DbAccess

###########################################################################
# GLOBALS

cdef object logger = logging.getLogger('retrieval')

# implemented as a singleton
cdef object dbaccess = DbAccess()

###########################################################################
# CLASS

cdef class TraceUpdater(object):
    """
    Manages the rankdata.

    .. note:: Rankdata has the size of the collection at hand.
    """

    #######################################################################
    # local functionality

    cdef dict _update_rankdata(self, dict rankdata):
        """
        Initialize the rankdata.

        FORM: {'imdbid': array(),
               'improb': array(),
               'tnclix': array(),}
        """

        logger.debug('START')
        start_time = now()

        if rankdata is None:
            # initialize rankdata
            rankdata = {}
            shape = (self.TRACE_LN,)
            rankdata['imdbid'] = self.dbimg.all_ids()

        else:
            # verify, then re-initialize rankdata
            if not isinstance(rankdata, dict):
                sys.stderr.write('\nUNKNOWN TYPE %s\n' % type(rankdata))
                sys.exit(1)

            if 'tnclix' in rankdata:
                sys.stderr.write('\nRANKCLIX OVERWRITTEN\n')
                sys.exit(1)

        # verify the rankdata
        if logger.getEffectiveLevel() < logging.DEBUG:
            for key, item in rankdata.iteritems():
                if len(item) != self.TRACE_LN:
                    sys.stderr.write('\nRANKDATA MISMATCH %s, %d, %d\n' %
                                     (key, len(item), self.TRACE_LN))
                    sys.exit(1)

        stop_time = now()
        logger.debug('DONE in %f seconds' % (stop_time - start_time))

        return rankdata

    #######################################################################
    #  main functionality

    cdef init(self):
        """
        Initialize the object.
        """

        self.dbimg = dbaccess.dbimg()
        self.TRACE_LN = self.dbimg.size()

    cdef dict rankdata(self, dict rankdata):
        """
        Initialize the rankdata.
        """

        rankdata = self._update_rankdata(rankdata)

        return rankdata

    cdef dict clean_rankdata(self, dict rankdata):
        """
        Clean the rankdata.
        """

        #if 'closestclix' in rankdata:
        del rankdata['closestclix']
        #if 'closestcldd' in rankdata:
        del rankdata['closestcldd']
        if 'sortedarg' in rankdata:
            del rankdata['sortedarg']
        if 'immass' in rankdata:
            del rankdata['immass']
        #if 'tnclix' in rankdata:
#        del rankdata['tnclix']

        return rankdata
