#!/usr/bin/env python

#cython: boundscheck=False
#cython: wraparound=False
#cython: embedsignature=True
#cython: profile=True

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Selects the image set for display.

.. note:: The images are chosen randomly.
"""

###########################################################################
# IMPORTS

import logging
from numpy import int32, float32, array, zeros, empty
from random import Random
import sys
from time import time as now

from retrieval_libs.common.candidate cimport skipped_images
from retrieval_libs.dblayer.DbAccess import DbAccess

###########################################################################
# GLOBALS

cdef object logger = logging.getLogger('retrieval')

# implemented as a singleton
cdef object dbaccess = DbAccess()

random = Random()
random.jumpahead(8)

###########################################################################
# CLASS

cdef class DisplaySetSelector(object):
    """
    Selects the image set for display.

    .. note:: The images are chosen randomly.
    """

    #######################################################################
    #  main functionality

    cdef init(self, cnp.int32_t size):
        """
        Initialize the object.
        """

        self.SIZE = size

    cdef dict update(self, dict rankdata, list dispfdbkH):
        """
        Return the image set for display, randomly chosen.
        """

        #cdef cnp.ndarray[cnp.int32_t, ndim=1, mode='c'] rankdbid = rankdata['imdbid']
        #cdef cnp.int32_t dbid

        logger.debug('START')
        start_time = now()

        # initialize seeddata
        seeddata = {}
        seeddata['imdbid'] = array([], dtype=int32, order='c')

        # initialize skipdisp
        skipdisp = skipped_images(dispfdbkH)

        # random seeddata
        # HINT: assumes that there is no missing id in the dbimg table
        dbimg = dbaccess.dbimg()
        #TRIX_MAX = len(rankdbid) - 1
        TRIX_MAX = dbimg.size()
        sddbid = []
        while len(sddbid) < self.SIZE:
            trix = random.randint(0, TRIX_MAX - 1)
            #dbid = rankdbid[trix]
            dbid = trix + 1
            if dbid not in sddbid and dbid not in skipdisp:
                valid = dbimg.im_valid((dbid,))
                if valid[0][1] == 1:
                    sddbid.append(dbid)

        seeddata['imdbid'] = empty((len(sddbid),), dtype=int32, order='c')
        for i, dbid in enumerate(sddbid):
            seeddata['imdbid'][i] = dbid

        seeddataln = len(seeddata['imdbid'])
        if seeddataln > 8:
            for key, item in seeddata.iteritems():
                # keep only the first 8
                seeddata[key] = item[:8]
                # keep only the last 8
                #seeddata[key] = item[-8:]

        stop_time = now()
        logger.debug('DONE in %f seconds' % (stop_time - start_time))

        return seeddata
