#!/usr/bin/env python

#cython: boundscheck=False
#cython: wraparound=False
#cython: embedsignature=True
#cython: profile=True

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Manages the random retrieval algorithm.
"""

###########################################################################
# IMPORTS

import logging
import sys

###########################################################################
# GLOBALS

cdef object logger = logging.getLogger('retrieval')

# the number of image/seeds selected for display
cdef cnp.int32_t SIZE = 8

###########################################################################
# CLASS

cdef class QueryImg(object):
    """
    Manages the random retrieval algorithm.
    """

    #######################################################################
    #  main functionality

    def init(self, SYS_THARGS, SYS_WEIGHTS, SYS_CONSISTENCY):
        """
        Initialize the object.
        """

        self.thargs = SYS_THARGS[0]
        self.weights = SYS_WEIGHTS[0]
        self.consistency = SYS_CONSISTENCY[0]

        self.rankdata = None
        self.rankdata_0 = None
        self.seeddata = None
        self.tracedata = None

        #self.dispfdbkH = None

        #self.traceupdater = TraceUpdater()
        #self.traceupdater.init()
        self.dispselector = DisplaySetSelector()
        self.dispselector.init(SIZE)

    def set_buffers(self):

        return

    def trim_buffers(self):

        return

    def set_data(self, dispfdbkH, thargsH, weightsH, consistencyH):

        self.dispfdbkH = dispfdbkH

    def trim_data(self):

        self.dispfdbkH = None

    #######################################################################
    # update functionality

    def update_thargs(self, rankdata, dispfdbk, dispfdbkH, thargsH, weightsH):
        """
        Update the thargs.
        """

        return self.thargs

    def update_weights(self, dispfdbk, dispfdbkH, thargsH, weightsH):
        """
        Update the weights.
        """

        return self.weights

    def update_consistency(self, rankdata, dispfdbk, dispfdbkH, thargsH, weightsH, consistencyH):
        """
        Update the consistency.
        """

        return self.consistency

    #######################################################################
    #  main functionality

    def perform_query_random(self, rankdata, rankdata_0, dispfdbk):
        """
        Generate the seeddata (ie. the next dispdata)
        - NO relevance feedback >> NO update for the image probabilities;
        - Voronoi partitioning starting from a random image/seed;
        """

        self.rankdata = rankdata
        #self.rankdata = self.traceupdater.rankdata(rankdata)

        self.seeddata = self.dispselector.update(self.rankdata, self.dispfdbkH)

    def perform_query_fdbk(self, rankdata, rankdata_0, dispfdbk):
        """
        Generate the seeddata (ie. the next dispdata)
        - relevance feedback >> update the image probabilities;
        - Voronoi partitioning starting from the image/seed with the highest probability;
        """

        self.rankdata = rankdata
        #self.rankdata = self.traceupdater.rankdata(rankdata)

        self.seeddata = self.dispselector.update(self.rankdata, self.dispfdbkH)

    #######################################################################
    # expand/collapse functionality

    def collapse_rankdata(self, rankdata):
        """
        Miss-used little bit...
        """

        #rankdata = self.traceupdater.clean_rankdata(rankdata)

        return rankdata
