#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Common settings.

- It configures the Django project and its applications
- It configures the loggers and sets them at the INFO level
"""

###########################################################################
# IMPORTS

import logging
import os
import socket
import sys
import warnings

from django.template.loader import add_to_builtins

###########################################################################

DEBUG = False
TEMPLATE_DEBUG = DEBUG

# create the logging object
logger = logging.getLogger('retrieval')
#if not logger.handlers: # for avoiding duplicate handlers
#create console handler
handler = logging.StreamHandler()
#create formatter
# %(levelname)s %(asctime)s %(funcName)s %(lineno)d %(message)s %(pathname)s %(filename)s')
formatter = logging.Formatter('%(levelname)s:%(name)s:%(module)s:%(funcName)s:%(message)s')
#add formatter to handler
handler.setFormatter(formatter)
#add formatter to logger
logger.addHandler(handler)
# set the level for all handlers
logger.setLevel(logging.INFO)

# REMINDER: useful calls
#logger.isEnabledFor(logging.DEBUG)
#logger.getEffectiveLevel() == logging.DEBUG

logger = logging.getLogger('assistant')
handler = logging.StreamHandler()
formatter = logging.Formatter('%(levelname)s:%(name)s:%(module)s:%(funcName)s:%(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)

logger = logging.getLogger('workflow')
handler = logging.StreamHandler()
formatter = logging.Formatter('%(levelname)s:%(name)s:%(module)s:%(funcName)s:%(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)


# ignore warnings for multiple imports
warnings.simplefilter('ignore')


ADMINS = (
    ('name', 'email@address'),
)

MANAGERS = ADMINS
SERVER_EMAIL = 'email@address'
SERVER_DOMAIN = 'site_www_address'

# for email
EMAIL_HOST = ''
EMAIL_USE_TLS = True


_DIR_CWD = os.path.dirname(__file__)
DIR_APP = os.path.abspath(os.path.join(_DIR_CWD, '..'))
DIR_DATA = os.path.abspath(os.path.join(_DIR_CWD, '../../media'))
DIR_TMP = os.path.abspath(os.path.join(_DIR_CWD, '../../tmp'))


if socket.gethostname() == 'linux_machine_name' or \
     os.getenv('SGE_TASK_ID') is not None:

    # Django webserver settings
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': '',
            'USER': '',
            'PASSWORD': '',
            'HOST': '',
            'PORT': '',
            'INTERFACE': 'MySQLdb',
        },
    }

    # list of system setups
    ALL_DB = {

        'ImageNet_1M': {
            # data files
            #'DIR_DIST': None,
            'TABLE': 'ImageNet_1M',
            #'DIR_IMG': None, # HINT: no prefix for the absolute urls
            'DEFAULT_CONFIG': 1,

            # tasks and configs for testing
            'TASKS': [],
            'CONFIGS': [],
        },

        'ImageNet_60K': {
            # data files
            'DIR_DIST': 'ImageNet/ImageNet_60K/distances',
            'TABLE': 'ImageNet_60K',
            #'DIR_IMG': None, # HINT: no prefix for the absolute urls
            'DEFAULT_CONFIG': 1,

            # tasks and configs for testing
            'TASKS': [],
            'CONFIGS': [],
        },

        'ImageNet_33K': {
            # data files
            'DIR_DIST': 'ImageNet/ImageNet_33K/distances',
            'TABLE': 'ImageNet_33K',
            #'DIR_IMG': None, # HINT: no prefix for the absolute urls
            'DEFAULT_CONFIG': 1,

            # tasks and configs for testing
            'TASKS': [],
            'CONFIGS': [],
        },

        'ImageSynthetic_22K': {
            # data files
            'DIR_DIST': 'ImageSynthetic/ImageSynthetic_22K/distances',
            'TABLE': 'ImageSynthetic_22K',
            'DIR_IMG': 'ImageSynthetic/ImageSynthetic_22K/data',
            'DEFAULT_CONFIG': 1,

            # tasks and configs for testing
            'TASKS': [],
            'CONFIGS': [],
        },
    }

    # default system setup
    DEFAULT_DB = 'ImageNet_1M'

    # PYTHONPATH
    sys.path.append(os.path.join(DIR_APP, 'apps'))
    sys.path.append(os.path.join(DIR_APP, 'libs'))

    # configure the cache-per-site
    CACHE_BACKEND = 'file://' + DIR_TMP + '/django-cache'
    #CACHE_BACKEND = 'memcached://127.0.0.1:8000/'
    # configure the session engine (cache-per-session)
    SESSION_ENGINE = 'django.contrib.sessions.backends.file'
    #SESSION_ENGINE = 'django.contrib.sessions.backends.cache'
    SESSION_FILE_PATH = os.path.join(DIR_TMP, 'django-cache')

    os.environ['MPLCONFIGDIR'] = os.path.join(DIR_TMP, 'matplotlib')

else:
    # Apache webserver settings
    raise NotImplementedError

#: the maximum number of iterations for the sessions during testing
ITER_MAX = 20

#: enable/disable the trace representations for display
FLAG_TRACEGFX = True
FLAG_TRACEGFX = {}
FLAG_TRACEGFX['TRIX_2D'] = True
FLAG_TRACEGFX['PROB_2D'] = False

DPI_TRACEGFX = 80
DPI_STATGFX = 80

###
#TABLE_USER = 'auth_user'
TABLE_USER = 'registration_user'


CACHE_MIDDLEWARE_SECONDS = 60 * 10
CACHE_MIDDLEWARE_KEY_PREFIX = ''
CACHE_MIDDLEWARE_ANONYMOUS_ONLY = False

SESSION_SAVE_EVERY_REQUEST = True
SESSION_EXPIRE_AT_BROWSER_CLOSE = False
#SESSION_COOKIE_DOMAIN = ''
SESSION_COOKIE_NAME = 'heat'
SESSION_COOKIE_PATH = '/'
SESSION_COOKIE_SECURE = False

APPEND_SLASH = False
LOGIN_REDIRECT_URL = '/retrieval/help/'
LOGIN_URL = '/registration/login/'
LOGOUT_URL = '/registration/logout/'

IGNORABLE_404_URLS = ('favicon.ico', 'robots.txt',)

# Local time zone for this installation. Choices can be found here:
# http://www.postgresql.org/docs/8.1/static/datetime-keywords.html#DATETIME-TIMEZONE-SET-TABLE
# although not all variations may be possible on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/Zurich'

# Language code for this installation. All choices can be found here:
# http://www.w3.org/TR/REC-html40/struct/dirlang.html#langcodes
# http://blogs.law.harvard.edu/tech/stories/storyReader$15
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True
LOCALE_PATHS = (os.path.join(DIR_APP, 'locale'),)

# Absolute path to the directory that holds media.
MEDIA_ROOT = DIR_DATA

# Absolute path to the directory that holds static files.
#STATIC_ROOT = DIR_DATA

# URL prefix for the media served from MEDIA_ROOT.
MEDIA_URL = '/files/'

# URL prefix for the static files located in STATIC_ROOT.
#STATIC_URL = '/files/'

# URL prefix for admin media -- CSS, JavaScript and images.
# Make sure to use a trailing slash.
# ALIAS defined in APACHE config
ADMIN_MEDIA_PREFIX = '/media/'


# complete the DB data
for key, data in ALL_DB.iteritems():

    # if the directories are defined, they should be valid
    rpath_dist = data.get('DIR_DIST', None)
    if rpath_dist is not None:
        fullpath_dist = os.path.join(DIR_DATA, rpath_dist)
        data['DIR_DIST'] = fullpath_dist
        if not os.access(fullpath_dist, os.F_OK):
            sys.stderr.write('\nMISSING DIR %s\n' % fullpath_dist)
            #sys.exit(1)

    # directory of the image collection
    rpath_img = data.get('DIR_IMG', None)
    if rpath_img is not None:
        data['DIR_IMG'] = os.path.join(MEDIA_ROOT, rpath_img)
        data['IMG_URL'] = MEDIA_URL + rpath_img + '/'

    # names of the related tables
    table = data['TABLE']
    data['TABLE_CONFIG'] = table + '_config'
    data['TABLE_TASK'] = table + '_task'
    data['TABLE_SEARCH'] = table + '_search'
    data['TABLE_USER'] = TABLE_USER


# FIGURES options
FIGURES = {
    'root_dir': os.path.join(MEDIA_ROOT, 'figures'),
    'root_url': MEDIA_URL + 'figures',
    'auto_cleanup': False,
    }


# Make this unique, and don't share it with anybody.
SECRET_KEY = ''


# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
    #'django.template.loaders.eggs.load_template_source',
)

# templatetags added to all templates
add_to_builtins('django.templatetags.i18n')


MIDDLEWARE_CLASSES = (
    #'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.cache.CacheMiddleware',
    #'django.middleware.cache.FetchFromCacheMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    #'django.contrib.messages.middleware.MessageMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.doc.XViewMiddleware',
    # HEAT middleware
    'retrieval.middleware.DbSetup',
    'retrieval.middleware.CrawlerBlocker',
)

ROOT_URLCONF = 'urls'

TEMPLATE_DIRS = (
    # Don't forget to use absolute paths, not relative paths.
    #'django/contrib/admin/templates',
    os.path.join(DIR_APP, 'templates'),
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    #'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    #'django.core.context_processors.static',
    #'django.contrib.messages.context_processors.messages'
    'django.core.context_processors.request',
    # HEAT processors
    'retrieval.context_processors.current_page',
    'retrieval.context_processors.active_db',
    'retrieval.context_processors.img_url',
    'retrieval.context_processors.dbinfo',
    )

INSTALLED_APPS = (
    # django apps
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    #'django.contrib.messages',
    #'django.contrib.admin',
    # HEAT apps
    'registration',
    'retrieval',
    # other django apps
    'django_extensions',
)
