#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Settings for uploading the image collections in the database.
"""

###########################################################################
# IMPORTS

import logging
import os
import socket
import sys
import warnings

###########################################################################
# logging settings

logger = logging.getLogger('retrieval')
handler = logging.StreamHandler()
formatter = logging.Formatter('%(levelname)s:%(name)s:%(module)s:%(funcName)s:%(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)


# ignore warnings for multiple imports
warnings.simplefilter('ignore')

###########################################################################
# database settings

_DIR_CWD = os.path.dirname(__file__)
DIR_APP = os.path.abspath(os.path.join(_DIR_CWD, '..'))
DIR_DATA = os.path.abspath(os.path.join(_DIR_CWD, '../../media'))

if socket.gethostname() == 'linux_machine_name' or \
     os.getenv('SGE_TASK_ID') is not None:
    # development settings

    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': '',
            'USER': '',
            'PASSWORD': '',
            'HOST': '',
            'PORT': '',
            'INTERFACE': 'MySQLdb',
        },
    }

    # list of system setups
    ALL_DB = {
        'ImageNet_1M': {
            # data files
            #'DIR_DIST': None,
            'TABLE': 'ImageNet_1M',
            #'DIR_IMG': None, # HINT: no prefix for the absolute urls
            'DEFAULT_CONFIG': 1,
        },

        'ImageNet_60K': {
            # data files
            'DIR_DIST': 'ImageNet/ImageNet_60K/distances',
            'TABLE': 'ImageNet_60K',
            #'DIR_IMG': None, # HINT: no prefix for the absolute urls
            'DEFAULT_CONFIG': 1,
        },

        'ImageNet_33K': {
            # data files
            'DIR_DIST': 'ImageNet/ImageNet_33K/distances',
            'TABLE': 'ImageNet_33K',
            #'DIR_IMG': None, # HINT: no prefix for the absolute urls
            'DEFAULT_CONFIG': 1,
        },

        'ImageSynthetic_22K': {
            # data files
            'DIR_DIST': 'ImageSynthetic/ImageSynthetic_22K/distances',
            'TABLE': 'ImageSynthetic_22K',
            'DIR_IMG': 'ImageSynthetic/ImageSynthetic_22K/data',
            'DEFAULT_CONFIG': 1,
        },
    }

    # default system setup
    DEFAULT_DB = ''

    # PYTHONPATH
    sys.path.append(os.path.join(DIR_APP, 'apps'))
    sys.path.append(os.path.join(DIR_APP, 'libs'))

else:
    # webserver settings
    raise NotImplementedError


###
#TABLE_USER = 'auth_user'
TABLE_USER = 'registration_user'

# Absolute path to the directory that holds media.
MEDIA_ROOT = DIR_DATA

# URL prefix for the media served from MEDIA_ROOT.
MEDIA_URL = '/files/'


# complete the DB data
for key, data in ALL_DB.iteritems():

    # if the directories are defined, they should be valid
    rpath_dist = data.get('DIR_DIST', None)
    if rpath_dist is not None:
        fullpath_dist = os.path.join(DIR_DATA, rpath_dist)
        data['DIR_DIST'] = fullpath_dist
        if not os.access(fullpath_dist, os.F_OK):
            sys.stderr.write('\nMISSING DIR %s\n' % fullpath_dist)
            #sys.exit(1)

    # directory of the image collection
    rpath_img = data.get('DIR_IMG', None)
    if rpath_img is not None:
        data['DIR_IMG'] = os.path.join(MEDIA_ROOT, rpath_img)
        data['IMG_URL'] = MEDIA_URL + rpath_img + '/'

    # names of the related tables
    table = data['TABLE']
    data['TABLE_CONFIG'] = table + '_config'
    data['TABLE_TASK'] = table + '_task'
    data['TABLE_SEARCH'] = table + '_search'
    data['TABLE_USER'] = TABLE_USER


# FIGURES options
FIGURES = {
    'root_dir': os.path.join(MEDIA_ROOT, 'figures'),
    'root_url': MEDIA_URL + 'figures',
    'auto_cleanup': False,
    }
