#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Settings for running the application programatically/automatically.
"""

###########################################################################
# IMPORTS

import logging
import os
import sys

# HINT: do NOT change the order of imports
from heat.settings.common import *
# WARNING: Running the automatic application in the debug mode is not possible
# when executed on the computing grid, but only when executed on the desktop.
#from heat.settings.debug import *

###########################################################################

if socket.gethostname() == 'linux_machine_name':
    # desktop settings
    AUTOUSR_NAME = 'autousr'
    AUTOUSR_PWD = 'pwd4autousr'

    # clean the DB data
    for key, data in ALL_DB.iteritems():
        del data['TASKS'][:]
        del data['CONFIGS'][:]

    ALL_DB['ImageNet_1M']['DEFAULT_CONFIG'] = 1
    ALL_DB['ImageNet_1M']['TASKS'] = [1, 2, 3, 4, 5, 6,]
    ALL_DB['ImageNet_1M']['CONFIGS'] = [1,]

    #: the number of tests to be performed by the automatic users
    #: ::
    #:
    #:     A **test** is a set of sessions. A **session** refers to one
    #:     search/retrieval process. Each session is performed on one
    #:     image database and it is configured with one task-config peer.
    #:
    #:     The number of sessions in one *test* is equal to the sum of
    #:     all possible task-config peers for all image databases.
    TESTS_MAX = 1000

    #: the period of the re-initialization of the automatic tasks
    #: ::
    #:
    #:     If it is set to TESTS_MAX, the tasks will be fixed once for all
    #:     the tests. This is useful for getting the statistical performance
    #:     of the automatic user for one specific task.
    #:
    #:     If it is set to 1, the tasks will be randomized after each test, so
    #:     after each complete set of sessions. This is useful for getting the
    #:     statistical performance of the automatic user for a class of tasks.
    REINIT_LOOP = 1

    #: enable/disable the trace representation for display
    FLAG_TRACEGFX = False

    DPI_TRACEGFX = 600
    DPI_STATGFX = 600

    #: for profiling/timing the program execution
    PROFILE = False
    PROFILE_FILE = DIR_TMP + '/profile/wprofile'

elif os.getenv('SGE_TASK_ID'):
    # grid settings

    #logger = logging.getLogger('retrieval')
    #logger.setLevel(logging.ERROR)

    #logger = logging.getLogger('assistant')
    #logger.setLevel(logging.ERROR)

    #logger = logging.getLogger('workflow')
    #logger.setLevel(logging.INFO)

    job_id = int(os.getenv('SGE_TASK_ID'))

    AUTOUSR_NAME = 'autousr_%d' % job_id
    AUTOUSR_PWD = 'pwd4autousr'

    # clean the DB data
    for key, data in ALL_DB.iteritems():
        del data['TASKS'][:]
        del data['CONFIGS'][:]
        
    ALL_DB['ImageNet_1M']['TASKS'] = [(job_id + 0),]
    ALL_DB['ImageNet_1M']['CONFIGS'] = [1,]

    TESTS_MAX = 1000
    REINIT_LOOP = 1

    FLAG_TRACEGFX = False

    PROFILE = False
    PROFILE_FILE = DIR_TMP + '/profile/wprofile_%d' % job_id

else:
    # webserver settings
    raise NotImplementedError
