#!/usr/bin/python

###########################################################################
#                                                                         #
# Copyright (c) 2011                   - Idiap Research Institute         #
#                                        http://www.idiap.ch/             #
#                                                                         #
# See file COPYING for the license associated to this software            #
#                                                                         #
###########################################################################

"""
Compile the Cython optimized code.

It compiles from scratch the *.pxd and *.pyx source files.

HINT: The compile command is::

    python setup.py build_ext --inplace
"""

###########################################################################
# IMPORTS

import os
import sys

from distutils.core import setup
from distutils.extension import Extension

try:
    from Cython.Distutils import build_ext
except:
    sys.stderr.write('\nCython is NOT installed\n\n')
    sys.exit(1)

import numpy

###########################################################################
# GLOBALS

DIR_CWD = os.path.dirname(__file__)
DIR_LIBS = os.path.abspath(os.path.join(DIR_CWD, 'heat/libs'))

###########################################################################

include_dirs = [numpy.get_include(), DIR_LIBS]

ext_modules = [

    Extension('heat.libs.retrieval_libs.common.buffer',
              ['heat/libs/retrieval_libs/common/buffer.pxd',
               'heat/libs/retrieval_libs/common/buffer.pyx',],
               include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.common.candidate',
              ['heat/libs/retrieval_libs/common/candidate.pxd',
               'heat/libs/retrieval_libs/common/candidate.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.common.extend',
              ['heat/libs/retrieval_libs/common/extend.pxd',
               'heat/libs/retrieval_libs/common/extend.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.common.index',
              ['heat/libs/retrieval_libs/common/index.pxd',
               'heat/libs/retrieval_libs/common/index.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.common.probability',
              ['heat/libs/retrieval_libs/common/probability.pxd',
               'heat/libs/retrieval_libs/common/probability.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.common.thdist',
              ['heat/libs/retrieval_libs/common/thdist.pxd',
               'heat/libs/retrieval_libs/common/thdist.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.dblayer.DbTable',
              ['heat/libs/retrieval_libs/dblayer/DbTable.pxd',
               'heat/libs/retrieval_libs/dblayer/DbTable.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.dblayer.DbTableImg',
              ['heat/libs/retrieval_libs/dblayer/DbTableImg.pxd',
               'heat/libs/retrieval_libs/dblayer/DbTableImg.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.dblayer.DbTableTree',
              ['heat/libs/retrieval_libs/dblayer/DbTableTree.pxd',
               'heat/libs/retrieval_libs/dblayer/DbTableTree.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.dblayer.DbAccess',
              ['heat/libs/retrieval_libs/dblayer/DbAccess.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.assist.Behavior',
              ['heat/libs/retrieval_libs/assist/Behavior.pxd',
               'heat/libs/retrieval_libs/assist/Behavior.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.dbutils.dbimg_distdata',
              ['heat/libs/retrieval_libs/dbutils/dbimg_distdata.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.dbutils.dbtree_create',
              ['heat/libs/retrieval_libs/dbutils/dbtree_create.pyx',],
              include_dirs=include_dirs),

    # SYSTEM 0/ORIG
    Extension('heat.libs.retrieval_libs.search.im_orig.ConsistencyUpdater',
              ['heat/libs/retrieval_libs/search/im_orig/ConsistencyUpdater.pxd',
               'heat/libs/retrieval_libs/search/im_orig/ConsistencyUpdater.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.search.im_orig.ProbabilityUpdater',
              ['heat/libs/retrieval_libs/search/im_orig/ProbabilityUpdater.pxd',
               'heat/libs/retrieval_libs/search/im_orig/ProbabilityUpdater.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.search.im_orig.TraceUpdater',
              ['heat/libs/retrieval_libs/search/im_orig/TraceUpdater.pxd',
               'heat/libs/retrieval_libs/search/im_orig/TraceUpdater.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.search.im_orig.DisplaySetSelector',
              ['heat/libs/retrieval_libs/search/im_orig/DisplaySetSelector.pxd',
               'heat/libs/retrieval_libs/search/im_orig/DisplaySetSelector.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.search.im_orig.QueryImg',
              ['heat/libs/retrieval_libs/search/im_orig/QueryImg.pxd',
               'heat/libs/retrieval_libs/search/im_orig/QueryImg.pyx',],
              include_dirs=include_dirs),

    # SYSTEM 2/HEAT
    Extension('heat.libs.retrieval_libs.search.im_heat.TraceUpdater',
              ['heat/libs/retrieval_libs/search/im_heat/TraceUpdater.pxd',
               'heat/libs/retrieval_libs/search/im_heat/TraceUpdater.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.search.im_heat.DisplaySetSelector',
              ['heat/libs/retrieval_libs/search/im_heat/DisplaySetSelector.pxd',
               'heat/libs/retrieval_libs/search/im_heat/DisplaySetSelector.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.search.im_heat.QueryImg',
              ['heat/libs/retrieval_libs/search/im_heat/QueryImg.pxd',
               'heat/libs/retrieval_libs/search/im_heat/QueryImg.pyx',],
              include_dirs=include_dirs),

    # SYSTEM -1/RND
    Extension('heat.libs.retrieval_libs.search.im_rnd.TraceUpdater',
              ['heat/libs/retrieval_libs/search/im_rnd/TraceUpdater.pxd',
               'heat/libs/retrieval_libs/search/im_rnd/TraceUpdater.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.search.im_rnd.DisplaySetSelector',
              ['heat/libs/retrieval_libs/search/im_rnd/DisplaySetSelector.pxd',
               'heat/libs/retrieval_libs/search/im_rnd/DisplaySetSelector.pyx',],
              include_dirs=include_dirs),

    Extension('heat.libs.retrieval_libs.search.im_rnd.QueryImg',
              ['heat/libs/retrieval_libs/search/im_rnd/QueryImg.pxd',
               'heat/libs/retrieval_libs/search/im_rnd/QueryImg.pyx',],
              include_dirs=include_dirs),
]

###########################################################################

setup(
    name='django-heat',
    version='1.0',
    description='Content-based image retrieval application',
    author='Nicolae Suditu',
    author_email='nicolae.suditu@idiap.ch',
    url='http://imr.idiap.ch/',
    license='GPLv3 License',
    keywords='content-based image retrieval'.split(),
    cmdclass={'build_ext': build_ext},
    ext_modules = ext_modules,
    platforms='Linux',
    long_description='''HEAT is a content-based image retrieval application.''',
    )
